@@ -1,4 +1,4 @@
-/* $Id: asapinstance.c 1730 2008-02-14 10:53:34Z dreibh $
+/* $Id: asapinstance.c 2076 2008-12-28 15:52:34Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -9,7 +9,7 @@
  *
  * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
- * Copyright (C) 2002-2008 by Thomas Dreibholz
+ * Copyright (C) 2002-2009 by Thomas Dreibholz
  *
  * Acknowledgements:
  * Realized in co-operation between Siemens AG and
@@ -290,7 +290,7 @@ static void asapInstanceConfigure(struct ASAPInstance* asapInstance,
 static bool asapInstanceConnectToRegistrar(struct ASAPInstance* asapInstance,
                                            int                  sd)
 {
-#ifdef sctp_sack_info
+#ifdef HAVE_SCTP_DELAYED_SACK
    struct sctp_sack_info sctpSACKInfo;
 #endif
 
@@ -329,7 +329,7 @@ static bool asapInstanceConnectToRegistrar(struct ASAPInstance* asapInstance,
       fprintf(stdlog, "Connected to registrar $%08x\n", asapInstance->RegistrarIdentifier);
       LOG_END
 
-#ifdef sctp_sack_info
+#ifdef HAVE_SCTP_DELAYED_SACK
       /* ====== Tune SACK handling ======================================= */
       sctpSACKInfo.sack_assoc_id = 0;
       sctpSACKInfo.sack_delay    = 0;
@@ -1002,6 +1002,9 @@ static void asapInstanceHandleEndpointKeepAlive(
             if(asapInstanceConnectToRegistrar(asapInstance, sd) == true) {
                /* The new registrar ID is already known */
                asapInstance->RegistrarIdentifier = message->RegistrarIdentifier;
+               LOG_NOTE
+               fprintf(stdlog, "Home registrar is $%08x\n", asapInstance->RegistrarIdentifier);
+               LOG_END
             }
          }
          else {
