@@ -1,4 +1,4 @@
-/* $Id: breakdetector.c 1726 2008-02-14 09:56:55Z dreibh $
+/* $Id: breakdetector.c 2086 2009-01-07 10:29:41Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -9,7 +9,7 @@
  *
  * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
- * Copyright (C) 2004-2008 by Thomas Dreibholz
+ * Copyright (C) 2004-2009 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -99,8 +99,9 @@ void uninstallBreakDetector()
    PrintedKill   = false;
    LastDetection = (unsigned long long)-1;
 #endif
-   DetectedBreak = false;
-   PrintedBreak  = false;
+   /* No reset here!
+      DetectedBreak = false;
+      PrintedBreak  = false; */
    Quiet         = false;
 }
 
