@@ -1,15 +1,34 @@
-/*
-   ###########################################################################
-   rspsim - An Open Source RSerPool Simulation for OMNeT++
-   Copyright (C) 2003-2004 Thomas Dreibholz
+// $Id: combinesummaries.cc 2086 2009-01-07 10:29:41Z dreibh $
+// ###########################################################################
+//             Thomas Dreibholz's R Simulation Scripts Collection
+//                  Copyright (C) 2004-2009 Thomas Dreibholz
+//
+//           Author: Thomas Dreibholz, dreibh@iem.uni-due.de
+// ###########################################################################
+//
+// This program is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// This program is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY// without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+//
+// Contact: dreibh@iem.uni-due.de
 
-   Author: Thomas Dreibholz, dreibh@exp-math.uni-essen.de
-   ###########################################################################
-*/
+#include <stdio.h>
+#include <stdlib.h>
+#include <unistd.h>
+#include <string.h>
+#include <bzlib.h>
 
 #include <iostream>
 #include <fstream>
-#include <bzlib.h>
 
 
 using namespace std;
