@@ -1,4 +1,4 @@
-/* $Id: fractalgeneratorservice.cc 1667 2008-02-04 10:12:05Z dreibh $
+/* $Id: fractalgeneratorservice.cc 2076 2008-12-28 15:52:34Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -9,7 +9,7 @@
  *
  * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
- * Copyright (C) 2002-2008 by Thomas Dreibholz
+ * Copyright (C) 2002-2009 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -65,7 +65,7 @@ FractalGeneratorServer::~FractalGeneratorServer()
 
 
 // ###### Create a FractalGenerator thread ##################################
-TCPLikeServer* FractalGeneratorServer::fractalGeneratorServerFactory(int sd, void* userData)
+TCPLikeServer* FractalGeneratorServer::fractalGeneratorServerFactory(int sd, void* userData, const uint32_t peIdentifier)
 {
    return(new FractalGeneratorServer(sd, (FractalGeneratorServerSettings*)userData));
 }
@@ -77,11 +77,11 @@ void FractalGeneratorServer::fractalGeneratorPrintParameters(const void* userDat
    const FractalGeneratorServerSettings* settings = (const FractalGeneratorServerSettings*)userData;
 
    puts("Fractal Generator Parameters:");
-   printf("   Data Max Time           = %u [ms]\n", (unsigned int)(settings->DataMaxTime / 1000ULL));
-   printf("   Cookie Max Time         = %u [ms]\n", (unsigned int)(settings->CookieMaxTime / 1000ULL));
-   printf("   Cookie Max Packets      = %u [Packets]\n", (unsigned int)settings->CookieMaxPackets);
-   printf("   Transmit Timeout        = %u [ms]\n", (unsigned int)settings->TransmitTimeout);
-   printf("   Failure After           = %u [Packets]\n", (unsigned int)settings->FailureAfter);
+   printf("   Transmit Timeout        = %ums\n", (unsigned int)settings->TransmitTimeout);
+   printf("   Data Max Time           = %ums\n", (unsigned int)(settings->DataMaxTime / 1000ULL));
+   printf("   Cookie Max Time         = %ums\n", (unsigned int)(settings->CookieMaxTime / 1000ULL));
+   printf("   Cookie Max Packets      = %u Packets\n", (unsigned int)settings->CookieMaxPackets);
+   printf("   Failure After           = %u Packets\n", (unsigned int)settings->FailureAfter);
    printf("   Test Mode               = %s\n", (settings->TestMode == true) ? "on" : "off");
 }
 
