@@ -1,41 +1,40 @@
-/*
- *  $Id: loglevel.h 1284 2007-01-10 18:41:25Z dreibh $
+/* $Id: loglevel.h 2076 2008-12-28 15:52:34Z dreibh $
+ * --------------------------------------------------------------------------
  *
- * RSerPool implementation.
+ *              //===//   //=====   //===//   //       //   //===//
+ *             //    //  //        //    //  //       //   //    //
+ *            //===//   //=====   //===//   //       //   //===<<
+ *           //   \\         //  //        //       //   //    //
+ *          //     \\  =====//  //        //=====  //   //===//    Version II
  *
- * Realized in co-operation between Siemens AG
- * and University of Essen, Institute of Computer Networking Technology.
+ * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
- * Acknowledgement
+ * Copyright (C) 2002-2009 by Thomas Dreibholz
+ *
+ * Acknowledgements:
+ * Realized in co-operation between Siemens AG and
+ * University of Essen, Institute of Computer Networking Technology.
  * This work was partially funded by the Bundesministerium fuer Bildung und
- * Forschung (BMBF) of the Federal Republic of Germany (Foerderkennzeichen 01AK045).
+ * Forschung (BMBF) of the Federal Republic of Germany
+ * (Förderkennzeichen 01AK045).
  * The authors alone are responsible for the contents.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- *
- * There are two mailinglists available at http://www.sctp.de/rserpool.html
- * which should be used for any discussion related to this implementation.
- *
- * Contact: rsplib-discussion@sctp.de
- *          dreibh@exp-math.uni-essen.de
- *
- * Purpose: Logging Management
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
+ * Contact: dreibh@iem.uni-due.de
  */
 
-
 #ifndef LOGLEVEL_H
 #define LOGLEVEL_H
 
@@ -48,6 +47,9 @@
 #include "threadsafety.h"
 #include "debug.h"
 
+#include <errno.h>
+#include <unistd.h>
+
 
 #ifdef __cplusplus
 extern "C" {
@@ -87,9 +89,9 @@ void setLogColor(const unsigned int color);
       setLogColor(c1);                         \
       printTimeStamp(stdlog);                  \
       setLogColor(c2);                         \
-      fprintf(stdlog,"P%u.%x@%s %s:%u %s()\n", \
-              (unsigned int)getpid(),          \
-              (unsigned int)pthread_self(),    \
+      fprintf(stdlog,"P%lu.%lx@%s %s:%u %s()\n", \
+              (unsigned long)getpid(),           \
+              (unsigned long)pthread_self(),     \
               getHostName(),                   \
               __FILE__,                        \
               __LINE__,                        \
