@@ -1,25 +1,30 @@
-/*
- * An Efficient RSerPool Pool Handlespace Management Implementation
- * Copyright (C) 2004-2007 by Thomas Dreibholz
+/* $Id: peerlistnode-template.h 2086 2009-01-07 10:29:41Z dreibh $
+ * --------------------------------------------------------------------------
  *
- * $Id: peerlistnode-template.h 1284 2007-01-10 18:41:25Z dreibh $
+ *              //===//   //=====   //===//   //=====  //   //      //
+ *             //    //  //        //    //  //       //   //=/  /=//
+ *            //===//   //=====   //===//   //====   //   //  //  //
+ *           //   \\         //  //             //  //   //  //  //
+ *          //     \\  =====//  //        =====//  //   //      //  Version V
  *
- * This program is free software; you can redistribute it and/or modify
+ * ------------- An Open Source RSerPool Simulation for OMNeT++ -------------
+ *
+ * Copyright (C) 2003-2009 by Thomas Dreibholz
+ *
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
 
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
-
+ *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- *
- * Contact: dreibh@exp-math.uni-essen.de
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
+ * Contact: dreibh@iem.uni-due.de
  */
 
 #ifndef INTERNAL_POOLTEMPLATE
@@ -40,12 +45,12 @@ struct TakeoverProcess;
 #define PLNF_STATIC    0
 #define PLNF_DYNAMIC   (1 << 0)
 #define PLNF_FROM_PEER (1 << 1)
-#define PLNF_MULTICAST (1 << 2)
+#define PLNF_NEW       (1 << 15)  /* Indicates that registration added new node */
 
 /* Status */
-#define PLNS_LISTSYNC  (1 << 0)   /* Peer List synchronization in progress    */
-#define PLNS_HTSYNC    (1 << 1)   /* Handle Table synchronization in progress */
-#define PLNS_MENTOR    (1 << 2)   /* Synchronization with mentor PR           */
+#define PLNS_LISTSYNC  (1 << 0)   /* Peer List synchronization in progress      */
+#define PLNS_HTSYNC    (1 << 1)   /* Handle Table synchronization in progress   */
+#define PLNS_MENTOR    (1 << 2)   /* Synchronization with mentor PR             */
 
 /* Timer Codes */
 #define PLNT_MAX_TIME_LAST_HEARD  3000
