@@ -1,4 +1,4 @@
-/* $Id: peerlistnode-template_impl.h 1667 2008-02-04 10:12:05Z dreibh $
+/* $Id: peerlistnode-template_impl.h 2086 2009-01-07 10:29:41Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //=====  //   //      //
@@ -9,7 +9,7 @@
  *
  * ------------- An Open Source RSerPool Simulation for OMNeT++ -------------
  *
- * Copyright (C) 2003-2008 by Thomas Dreibholz
+ * Copyright (C) 2003-2009 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -184,7 +184,7 @@ void ST_CLASS(peerListNodeGetDescription)(
             peerListNode->Identifier,
             peerListNode->LastUpdateTimeStamp,
             (unsigned int)handlespaceChecksumFinish(peerListNode->OwnershipChecksum));
-   if(peerListNode->Flags & PENF_NEW) {
+   if(peerListNode->Flags & PLNF_NEW) {
       safestrcat(buffer, "[new]", bufferSize);
    }
    if(!(peerListNode->Flags & PLNF_DYNAMIC)) {
