@@ -1,4 +1,4 @@
-/* $Id: poolhandlespacemanagement-basics.c 1667 2008-02-04 10:12:05Z dreibh $
+/* $Id: poolhandlespacemanagement-basics.c 2086 2009-01-07 10:29:41Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //=====  //   //      //
@@ -9,7 +9,7 @@
  *
  * ------------- An Open Source RSerPool Simulation for OMNeT++ -------------
  *
- * Copyright (C) 2003-2008 by Thomas Dreibholz
+ * Copyright (C) 2003-2009 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -48,3 +48,28 @@ PoolElementIdentifierType getPoolElementIdentifier()
    PoolElementIdentifierType poolElementIdentifier = 1 + (random32() % 0xfffffffe);
    return(poolElementIdentifier);
 }
+
+
+/* ###### Compute a hash from PH and PE ID ############################### */
+unsigned int computePHPEHash(const struct PoolHandle*        poolHandle,
+                             const PoolElementIdentifierType identifier)
+{
+   uint32_t hash = 0;
+   uint32_t rest;
+
+   const uint32_t* ph = (const uint32_t*)&poolHandle->Handle;
+   ssize_t i = poolHandle->Size;
+   while(i >= (ssize_t)sizeof(uint32_t)) {
+      hash = hash ^ *ph;
+      ph++;
+      i -= sizeof(uint32_t);
+   }
+   if(i > 0) {
+      rest = 0;
+      memcpy(&rest, ph, i);
+      hash = hash ^ rest;
+   }
+
+   hash = hash ^ (uint32_t)identifier;
+   return(hash);
+}
