@@ -1,25 +1,30 @@
-/*
- * An Efficient RSerPool Pool Handlespace Management Implementation
- * Copyright (C) 2004-2007 by Thomas Dreibholz
+/* $Id: poolnode-template_impl.h 2086 2009-01-07 10:29:41Z dreibh $
+ * --------------------------------------------------------------------------
  *
- * $Id: poolnode-template_impl.h 1284 2007-01-10 18:41:25Z dreibh $
+ *              //===//   //=====   //===//   //=====  //   //      //
+ *             //    //  //        //    //  //       //   //=/  /=//
+ *            //===//   //=====   //===//   //====   //   //  //  //
+ *           //   \\         //  //             //  //   //  //  //
+ *          //     \\  =====//  //        =====//  //   //      //  Version V
  *
- * This program is free software; you can redistribute it and/or modify
+ * ------------- An Open Source RSerPool Simulation for OMNeT++ -------------
+ *
+ * Copyright (C) 2003-2009 by Thomas Dreibholz
+ *
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
 
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
-
+ *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- *
- * Contact: dreibh@exp-math.uni-essen.de
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
+ * Contact: dreibh@iem.uni-due.de
  */
 
 /* ###### Print ########################################################## */
@@ -81,9 +86,9 @@ void ST_CLASS(poolNodeGetDescription)(const struct ST_CLASS(PoolNode)* poolNode,
                                       char*                            buffer,
                                       const size_t                     bufferSize)
 {
-   char  poolHandleDescription[256];
-   char  poolDescription[128];
-   char* protocol = "(unknown)";
+   char        poolHandleDescription[256];
+   char        poolDescription[128];
+   const char* protocol = "(unknown)";
 
    switch(poolNode->Protocol) {
       case IPPROTO_SCTP:
@@ -253,18 +258,6 @@ struct ST_CLASS(PoolElementNode)* ST_CLASS(poolNodeGetFirstPoolElementNodeFromIn
 };
 
 
-/* ###### Get last PoolElementNode from Index ############################ */
-struct ST_CLASS(PoolElementNode)* ST_CLASS(poolNodeGetLastPoolElementNodeFromIndex)(
-                                     struct ST_CLASS(PoolNode)* poolNode)
-{
-   struct STN_CLASSNAME* node = ST_METHOD(GetLast)(&poolNode->PoolElementIndexStorage);
-   if(node) {
-      return(ST_CLASS(getPoolElementNodeFromPoolElementIndexStorageNode)(node));
-   }
-   return(NULL);
-};
-
-
 /* ###### Get next PoolElementNode from Index ############################ */
 struct ST_CLASS(PoolElementNode)* ST_CLASS(poolNodeGetNextPoolElementNodeFromIndex)(
                                      struct ST_CLASS(PoolNode)*        poolNode,
@@ -279,20 +272,6 @@ struct ST_CLASS(PoolElementNode)* ST_CLASS(poolNodeGetNextPoolElementNodeFromInd
 }
 
 
-/* ###### Get previous PoolElementNode from Index ######################## */
-struct ST_CLASS(PoolElementNode)* ST_CLASS(poolNodeGetPrevPoolElementNodeFromIndex)(
-                                     struct ST_CLASS(PoolNode)*        poolNode,
-                                     struct ST_CLASS(PoolElementNode)* poolElementNode)
-{
-   struct STN_CLASSNAME* node = ST_METHOD(GetPrev)(&poolNode->PoolElementIndexStorage,
-                                                   &poolElementNode->PoolElementIndexStorageNode);
-   if(node) {
-      return(ST_CLASS(getPoolElementNodeFromPoolElementIndexStorageNode)(node));
-   }
-   return(NULL);
-}
-
-
 /* ###### Get first PoolElementNode from Selection ####################### */
 struct ST_CLASS(PoolElementNode)* ST_CLASS(poolNodeGetFirstPoolElementNodeFromSelection)(
                                      struct ST_CLASS(PoolNode)* poolNode)
@@ -395,6 +374,7 @@ struct ST_CLASS(PoolElementNode)* ST_CLASS(poolNodeAddPoolElementNode)(
          poolNode->GlobalSeqNumber) {
          ST_CLASS(poolNodeResequence)(poolNode);
       }
+      poolElementNode->Flags |= PENF_UPDATED;
       poolElementNode->SeqNumber        = poolNode->GlobalSeqNumber++;
       poolElementNode->VirtualCounter   = 0;
       poolElementNode->RoundCounter     = 0;
