@@ -1,4 +1,4 @@
-/* $Id: randomizer.c 1726 2008-02-14 09:56:55Z dreibh $
+/* $Id: randomizer.c 2086 2009-01-07 10:29:41Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //=====  //   //      //
@@ -9,7 +9,7 @@
  *
  * ------------- An Open Source RSerPool Simulation for OMNeT++ -------------
  *
- * Copyright (C) 2003-2008 by Thomas Dreibholz
+ * Copyright (C) 2003-2009 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -90,7 +90,7 @@ uint32_t random32()
    const double value = uniform(0.0, (double)0xffffffff);
    return((uint32_t)rint(value));
 #else
-  uint32_t number;
+   uint32_t number;
 
    switch(RandomSource) {
       case RS_DEVICE:
@@ -123,7 +123,7 @@ uint32_t random32()
 /* ###### Get double random value ######################################## */
 double randomDouble()
 {
-   return( (double)random32() / (double)4294967296ULL );
+   return( (double)random32() / (double)4294967296.0 );
 }
 
 
