@@ -1,4 +1,4 @@
-/* $Id: registrar-asap.c 1863 2008-08-26 13:24:38Z dreibh $
+/* $Id: registrar-asap.c 2076 2008-12-28 15:52:34Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -9,7 +9,7 @@
  *
  * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
- * Copyright (C) 2002-2008 by Thomas Dreibholz
+ * Copyright (C) 2002-2009 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -50,8 +50,10 @@ void registrarHandleASAPAnnounceTimer(struct Dispatcher* dispatcher,
       message->RegistrarIdentifier = registrar->ServerID;
       messageLength = rserpoolMessage2Packet(message);
       if(messageLength > 0) {
+#ifdef ENABLE_REGISTRAR_STATISTICS
          registrarWriteActionLog(registrar, "Send", "MulticastASAP", "Announce", "", 0, 0, 0,
                                  NULL, 0, message->RegistrarIdentifier, 0, 0, 0);
+#endif
          if(sendmulticast(
                registrar->ASAPAnnounceSocket,
                registrar->ASAPAnnounceSocketFamily,
@@ -95,8 +97,10 @@ void registrarHandlePoolElementEvent(struct Dispatcher* dispatcher,
             &registrar->Handlespace.Handlespace,
             poolElementNode);
 
+#ifdef ENABLE_REGISTRAR_STATISTICS
          registrarWriteActionLog(registrar, "Send", "ASAP", "EndpointKeepAlive", "KeepAliveTransmissionTimer", 0, 0, registrar->EndpointKeepAliveTimeoutInterval,
                                  &poolElementNode->OwnerPoolNode->Handle,poolElementNode->Identifier, registrar->ServerID, 0, 0, 0);
+#endif
 
          registrarSendASAPEndpointKeepAlive(registrar, poolElementNode, false);
          poolElementNode->LastKeepAliveTransmission = getMicroTime();
@@ -116,9 +120,11 @@ void registrarHandlePoolElementEvent(struct Dispatcher* dispatcher,
             poolHandlePrint(&poolElementNode->OwnerPoolNode->Handle, stdlog);
             fputs(" -> removing it\n", stdlog);
             LOG_END
+#ifdef ENABLE_REGISTRAR_STATISTICS
             registrarWriteActionLog(registrar, "Send", "ASAP", "Deregistration", "KeepAliveTimeoutTimer", 0, 0, registrar->EndpointKeepAliveTimeoutInterval,
                                     &poolElementNode->OwnerPoolNode->Handle, poolElementNode->Identifier, registrar->ServerID, 0, 0, 0);
-         }
+#endif
+}
          else {
             LOG_ACTION
             fprintf(stdlog, "Expiry timeout expired for pool element $%08x of pool ",
@@ -126,8 +132,10 @@ void registrarHandlePoolElementEvent(struct Dispatcher* dispatcher,
             poolHandlePrint(&poolElementNode->OwnerPoolNode->Handle, stdlog);
             fputs(" -> removing it\n", stdlog);
             LOG_END
+#ifdef ENABLE_REGISTRAR_STATISTICS
             registrarWriteActionLog(registrar, "Send", "ASAP", "Deregistration", "ExpiryTimer", 0, 0, registrar->TakeoverExpiryInterval,
                                     &poolElementNode->OwnerPoolNode->Handle, poolElementNode->Identifier, registrar->ServerID, 0, 0, 0);
+#endif
          }
 
          if(poolElementNode->HomeRegistrarIdentifier == registrar->ServerID) {
@@ -209,8 +217,10 @@ void registrarRemovePoolElementsOfConnection(struct Registrar*  registrar,
             registrarSendENRPHandleUpdate(registrar, poolElementNode, PNUP_DEL_PE);
          }
 
+#ifdef ENABLE_REGISTRAR_STATISTICS
          registrarWriteActionLog(registrar, "Send", "ASAP", "Deregistration", "Disconnect", 0, 0, 0,
                                  &poolElementNode->OwnerPoolNode->Handle, poolElementNode->Identifier, registrar->ServerID, 0, 0, 0);
+#endif
 
          result = ST_CLASS(poolHandlespaceManagementDeregisterPoolElementByPtr)(
                      &registrar->Handlespace,
@@ -244,8 +254,8 @@ void registrarRemovePoolElementsOfConnection(struct Registrar*  registrar,
 
 /* ###### Handle ASAP Registration ####################################### */
 void registrarHandleASAPRegistration(struct Registrar*       registrar,
-                                     int                     fd,
-                                     sctp_assoc_t            assocID,
+                                     const int               fd,
+                                     const sctp_assoc_t      assocID,
                                      struct RSerPoolMessage* message)
 {
 
@@ -265,8 +275,10 @@ void registrarHandleASAPRegistration(struct Registrar*       registrar,
    fprintf(stdlog, "/$%08x\n", message->PoolElementPtr->Identifier);
    LOG_END
 
+#ifdef ENABLE_REGISTRAR_STATISTICS
    registrarWriteActionLog(registrar, "Recv", "ASAP", "Registration", "Requested", 0, 0, 0,
                            &message->Handle, message->PoolElementPtr->Identifier, registrar->ServerID, 0, 0, 0);
+#endif
 
    message->Type       = AHT_REGISTRATION_RESPONSE;
    message->Flags      = 0x00;
@@ -365,11 +377,13 @@ void registrarHandleASAPRegistration(struct Registrar*       registrar,
                      &registrar->Handlespace.Handlespace,
                      poolElementNode);
 
-                  registrar->ReregistrationCount++;   /* We have a re-registration here */
-               }
-               else {
-                  registrar->RegistrationCount++;   /* We have a new registration here */
+#ifdef ENABLE_REGISTRAR_STATISTICS
+                  registrar->Stats.ReregistrationCount++;   /* We have a re-registration here */
+#endif
                }
+#ifdef ENABLE_REGISTRAR_STATISTICS
+               registrar->Stats.RegistrationCount++;   /* New registration or re-registration */
+#endif
                ST_CLASS(poolHandlespaceNodeActivateTimer)(
                   &registrar->Handlespace.Handlespace,
                   poolElementNode,
@@ -434,12 +448,15 @@ void registrarHandleASAPRegistration(struct Registrar*       registrar,
       if(message->Error) {
          message->Flags |= AHF_REGISTRATION_REJECT;
       }
+#ifdef ENABLE_REGISTRAR_STATISTICS
       registrarWriteActionLog(registrar, "Recv", "ASAP", "RegistrationResponse", "Requested", 0, 0, 0,
                               &message->Handle, message->PoolElementPtr->Identifier, registrar->ServerID, 0, 0, message->Error);
+#endif
       if(rserpoolMessageSend(IPPROTO_SCTP, fd, assocID, 0, 0, 0, message) == false) {
          LOG_WARNING
          logerror("Sending RegistrationResponse failed");
          LOG_END
+         sendabort(fd, assocID);
       }
    }
    else {
@@ -453,8 +470,8 @@ void registrarHandleASAPRegistration(struct Registrar*       registrar,
 
 /* ###### Handle ASAP Deregistration ##################################### */
 void registrarHandleASAPDeregistration(struct Registrar*       registrar,
-                                       int                     fd,
-                                       sctp_assoc_t            assocID,
+                                       const int               fd,
+                                       const sctp_assoc_t      assocID,
                                        struct RSerPoolMessage* message)
 {
    struct ST_CLASS(PoolElementNode)* poolElementNode;
@@ -467,8 +484,10 @@ void registrarHandleASAPDeregistration(struct Registrar*       registrar,
    fprintf(stdlog, "/$%08x\n", message->Identifier);
    LOG_END
 
+#ifdef ENABLE_REGISTRAR_STATISTICS
    registrarWriteActionLog(registrar, "Send", "ASAP", "Deregistration", "Requested", 0, 0, 0,
                            &message->Handle, message->Identifier, registrar->ServerID, 0, 0, 0);
+#endif
 
    message->Type  = AHT_DEREGISTRATION_RESPONSE;
    message->Flags = 0x00;
@@ -513,7 +532,9 @@ void registrarHandleASAPDeregistration(struct Registrar*       registrar,
             registrarDumpHandlespace(registrar);
             LOG_END
 
-            registrar->DeregistrationCount++;
+#ifdef ENABLE_REGISTRAR_STATISTICS
+            registrar->Stats.DeregistrationCount++;
+#endif
          }
          else {
             LOG_WARNING
@@ -554,20 +575,23 @@ void registrarHandleASAPDeregistration(struct Registrar*       registrar,
       LOG_END
    }
 
+#ifdef ENABLE_REGISTRAR_STATISTICS
    registrarWriteActionLog(registrar, "Send", "ASAP", "DeregistrationResponse", "Requested", message->Flags, 0, 0,
                            &message->Handle, message->Identifier, registrar->ServerID, 0, 0, message->Error);
+#endif
    if(rserpoolMessageSend(IPPROTO_SCTP, fd, assocID, 0, 0, 0, message) == false) {
       LOG_WARNING
       logerror("Sending DeregistrationResponse failed");
       LOG_END
+      sendabort(fd, assocID);
    }
 }
 
 
 /* ###### Handle ASAP Handle Resolution ################################## */
 void registrarHandleASAPHandleResolution(struct Registrar*       registrar,
-                                         int                     fd,
-                                         sctp_assoc_t            assocID,
+                                         const int               fd,
+                                         const sctp_assoc_t      assocID,
                                          struct RSerPoolMessage* message)
 {
    struct ST_CLASS(PoolElementNode)* poolElementNodeArray[MAX_MAX_HANDLE_RESOLUTION_ITEMS];
@@ -588,21 +612,32 @@ void registrarHandleASAPHandleResolution(struct Registrar*       registrar,
            (unsigned int)items, (unsigned int)message->Addresses);
    LOG_END
 
+#ifdef ENABLE_REGISTRAR_STATISTICS
    registrarWriteActionLog(registrar, "Recv", "ASAP", "HandleResolution", "Requested", 0, items, 0,
                            &message->Handle, 0, 0, 0, 0, 0);
-
-
-   registrar->HandleResolutionCount++;
+   registrar->Stats.HandleResolutionCount++;
+#endif
 
    message->Type  = AHT_HANDLE_RESOLUTION_RESPONSE;
    message->Flags = 0x00;
-   message->Error = ST_CLASS(poolHandlespaceManagementHandleResolution)(
-                       &registrar->Handlespace,
-                       &message->Handle,
-                       (struct ST_CLASS(PoolElementNode)**)&poolElementNodeArray,
-                       &poolElementNodes,
-                       items,
-                       registrar->MaxIncrement);
+   if( (registrar->MaxHRRate > 0.0) &&
+       (!registrarPoolUserHasPermissionFor(registrar, fd, assocID, AHT_HANDLE_RESOLUTION,
+                                           &message->Handle, 0)) ) {
+      LOG_WARNING
+      fprintf(stdlog, "Refusing handle resolution for assoc %u\n", assocID);
+      LOG_END
+      poolElementNodes = 0;
+      message->Error   = RSPERR_NOT_FOUND;
+   }
+   else {
+      message->Error = ST_CLASS(poolHandlespaceManagementHandleResolution)(
+                          &registrar->Handlespace,
+                          &message->Handle,
+                          (struct ST_CLASS(PoolElementNode)**)&poolElementNodeArray,
+                          &poolElementNodes,
+                          items,
+                          registrar->MaxIncrement);
+   }
    if(message->Error == RSPERR_OKAY) {
       LOG_VERBOSE1
       fprintf(stdlog, "Selected %u element%s\n", (unsigned int)poolElementNodes,
@@ -639,22 +674,25 @@ void registrarHandleASAPHandleResolution(struct Registrar*       registrar,
       LOG_END
    }
 
+#ifdef ENABLE_REGISTRAR_STATISTICS
    registrarWriteActionLog(registrar, "Send", "ASAP", "HandleResolutionResponse", "Requested", 0, poolElementNodes, 0,
                            &message->Handle, (poolElementNodes > 0) ? poolElementNodeArray[0]->Identifier : 0,
                            0, 0, 0, message->Error);
+#endif
 
    if(rserpoolMessageSend(IPPROTO_SCTP, fd, assocID, 0, 0, 0, message) == false) {
       LOG_WARNING
       logerror("Sending handle resolution response failed");
       LOG_END
+      sendabort(fd, assocID);
    }
 }
 
 
 /* ###### Handle ASAP Endpoint Keep-Alive Ack ############################ */
 void registrarHandleASAPEndpointKeepAliveAck(struct Registrar*       registrar,
-                                             int                     fd,
-                                             sctp_assoc_t            assocID,
+                                             const int               fd,
+                                             const sctp_assoc_t      assocID,
                                              struct RSerPoolMessage* message)
 {
    struct ST_CLASS(PoolElementNode)* poolElementNode;
@@ -676,9 +714,11 @@ void registrarHandleASAPEndpointKeepAliveAck(struct Registrar*       registrar,
       fputs("EndpointKeepAlive successful, resetting timer\n", stdlog);
       LOG_END
 
+#ifdef ENABLE_REGISTRAR_STATISTICS
       registrarWriteActionLog(registrar, "Recv", "ASAP", "EndpointKeepAliveAck", "InTime", 0, 0,
                               now - poolElementNode->LastKeepAliveTransmission,
                               &message->Handle, message->Identifier, 0, 0, 0, 0);
+#endif
 
       ST_CLASS(poolHandlespaceNodeDeactivateTimer)(
          &registrar->Handlespace.Handlespace,
@@ -693,8 +733,10 @@ void registrarHandleASAPEndpointKeepAliveAck(struct Registrar*       registrar,
                       &registrar->Handlespace));
    }
    else {
+#ifdef ENABLE_REGISTRAR_STATISTICS
       registrarWriteActionLog(registrar, "Recv", "ASAP", "EndpointKeepAliveAck", "TooLateAlreadyRemoved", 0, 0, 0,
                               &message->Handle, message->Identifier, 0, 0, 0, 0);
+#endif
 
       LOG_WARNING
       fprintf(stdlog,
@@ -760,8 +802,13 @@ void registrarSendASAPEndpointKeepAlive(struct Registrar*                 regist
          poolHandlePrint(&poolElementNode->OwnerPoolNode->Handle, stdlog);
          fputs(" failed\n", stdlog);
          LOG_END
+         sendabort(registrar->ASAPSocket, 0);
       }
 
+#ifdef ENABLE_REGISTRAR_STATISTICS
+      registrar->Stats.EndpointKeepAliveCount++;
+#endif
+
       rserpoolMessageDelete(message);
    }
 }
@@ -769,8 +816,8 @@ void registrarSendASAPEndpointKeepAlive(struct Registrar*                 regist
 
 /* ###### Handle ASAP Endpoint Unreachable ############################### */
 void registrarHandleASAPEndpointUnreachable(struct Registrar*       registrar,
-                                            int                     fd,
-                                            sctp_assoc_t            assocID,
+                                            const int               fd,
+                                            const sctp_assoc_t      assocID,
                                             struct RSerPoolMessage* message)
 {
    struct ST_CLASS(PoolElementNode)* poolElementNode;
@@ -783,10 +830,23 @@ void registrarHandleASAPEndpointUnreachable(struct Registrar*       registrar,
    fputs("\n", stdlog);
    LOG_END
 
-   registrar->FailureReportCount++;
+   if( (registrar->MaxEURate > 0.0) &&
+       (!registrarPoolUserHasPermissionFor(registrar, fd, assocID, AHT_ENDPOINT_UNREACHABLE,
+                                           &message->Handle, 0  /* === only pool! === */)) ) {
+      LOG_WARNING
+      fprintf(stdlog, "Refusing EndpointUnreachable for pool element $%08x of pool ",
+              message->Identifier);
+      poolHandlePrint(&message->Handle, stdlog);
+      fprintf(stdlog, " from assoc %u\n", assocID);
+      LOG_END
+      return;
+   }
 
-   registrarWriteActionLog(registrar, "Recv", "ASAP", "EndpointUnreachable", "", 0, registrar->FailureReportCount, 0,
+#ifdef ENABLE_REGISTRAR_STATISTICS
+   registrarWriteActionLog(registrar, "Recv", "ASAP", "EndpointUnreachable", "", 0, registrar->Stats.FailureReportCount, 0,
                            &message->Handle, message->Identifier, 0, 0, 0, 0);
+   registrar->Stats.FailureReportCount++;
+#endif
 
    poolElementNode = ST_CLASS(poolHandlespaceManagementFindPoolElement)(
                         &registrar->Handlespace,
