@@ -1,4 +1,4 @@
-/* $Id: registrar-global.c 1744 2008-02-15 08:51:50Z dreibh $
+/* $Id: registrar-global.c 2076 2008-12-28 15:52:34Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -9,7 +9,7 @@
  *
  * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
- * Copyright (C) 2002-2008 by Thomas Dreibholz
+ * Copyright (C) 2002-2009 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -34,9 +34,11 @@ static void poolElementNodeDisposer(struct ST_CLASS(PoolElementNode)* poolElemen
                                     void*                             userData);
 static void peerListNodeDisposer(struct ST_CLASS(PeerListNode)* peerListNode,
                                  void*                          userData);
+#ifdef ENABLE_REGISTRAR_STATISTICS
 static void statisticsCallback(struct Dispatcher* dispatcher,
                                struct Timer*      timer,
                                void*              userData);
+#endif
 #ifdef ENABLE_CSP
 static size_t registrarGetReportFunction(
                  void*                         userData,
@@ -49,27 +51,34 @@ static size_t registrarGetReportFunction(
 
 
 /* ###### Constructor #################################################### */
-struct Registrar* registrarNew(const RegistrarIdentifierType  serverID,
-                               int                            asapUnicastSocket,
-                               int                            asapAnnounceSocket,
-                               int                            enrpUnicastSocket,
-                               int                            enrpMulticastOutputSocket,
-                               int                            enrpMulticastInputSocket,
-                               const bool                     asapSendAnnounces,
-                               const union sockaddr_union*    asapAnnounceAddress,
-                               const bool                     enrpAnnounceViaMulticast,
-                               const union sockaddr_union*    enrpMulticastAddress,
-                               FILE*                          statsFile,
-                               unsigned int                   statsInterval
+struct Registrar* registrarNew(const RegistrarIdentifierType serverID,
+                               int                           asapUnicastSocket,
+                               int                           asapAnnounceSocket,
+                               int                           enrpUnicastSocket,
+                               int                           enrpMulticastOutputSocket,
+                               int                           enrpMulticastInputSocket,
+                               const bool                    asapSendAnnounces,
+                               const union sockaddr_union*   asapAnnounceAddress,
+                               const bool                    enrpAnnounceViaMulticast,
+                               const union sockaddr_union*   enrpMulticastAddress
+#ifdef ENABLE_REGISTRAR_STATISTICS
+                             , FILE*                         actionLogFile,
+                               BZFILE*                       actionLogBZFile,
+                               FILE*                         statsFile,
+                               BZFILE*                       statsBZFile,
+                               const unsigned int            statsInterval,
+                               const bool                    needsWeightedStatValues
+#endif
 #ifdef ENABLE_CSP
-                               , const unsigned int           cspReportInterval,
-                               const union sockaddr_union*    cspReportAddress
+                             , const unsigned int            cspReportInterval,
+                               const union sockaddr_union*   cspReportAddress
 #endif
                                )
 {
    struct Registrar*     registrar;
    int                   autoCloseTimeout;
-#ifdef sctp_sack_info
+   int                   noDelayOn;
+#ifdef HAVE_SCTP_DELAYED_SACK
    struct sctp_sack_info sctpSACKInfo;
 #endif
 
@@ -105,6 +114,7 @@ struct Registrar* registrarNew(const RegistrarIdentifierType  serverID,
                                              NULL,
                                              poolElementNodeDisposer,
                                              registrar);
+      ST_CLASS(poolUserListNew)(&registrar->PoolUsers);
       ST_CLASS(peerListManagementNew)(&registrar->Peers,
                                       &registrar->Handlespace,
                                       registrar->ServerID,
@@ -127,18 +137,18 @@ struct Registrar* registrarNew(const RegistrarIdentifierType  serverID,
                registrarHandlePeerEvent,
                (void*)registrar);
 
-      registrar->InStartupPhase            = true;
-      registrar->MentorServerID            = 0;
+      registrar->InStartupPhase                = true;
+      registrar->MentorServerID                = 0;
 
-      registrar->ASAPSocket                = asapUnicastSocket;
-      registrar->ASAPAnnounceSocket        = asapAnnounceSocket;
-      registrar->ASAPSendAnnounces         = asapSendAnnounces;
-
-      registrar->ENRPUnicastSocket         = enrpUnicastSocket;
-      registrar->ENRPMulticastInputSocket  = enrpMulticastOutputSocket;
-      registrar->ENRPMulticastOutputSocket = enrpMulticastInputSocket;
-      registrar->ENRPAnnounceViaMulticast  = enrpAnnounceViaMulticast;
+      registrar->ASAPSocket                    = asapUnicastSocket;
+      registrar->ASAPAnnounceSocket            = asapAnnounceSocket;
+      registrar->ASAPSendAnnounces             = asapSendAnnounces;
 
+      registrar->ENRPUnicastSocket             = enrpUnicastSocket;
+      registrar->ENRPMulticastInputSocket      = enrpMulticastOutputSocket;
+      registrar->ENRPMulticastOutputSocket     = enrpMulticastInputSocket;
+      registrar->ENRPAnnounceViaMulticast      = enrpAnnounceViaMulticast;
+      registrar->ENRPSupportTakeoverSuggestion = REGISTRAR_DEFAULT_SUPPORT_TAKEOVER_SUGGESTION;
 
       registrar->DistanceStep                          = REGISTRAR_DEFAULT_DISTANCE_STEP;
       registrar->MaxBadPEReports                       = REGISTRAR_DEFAULT_MAX_BAD_PE_REPORTS;
@@ -156,18 +166,36 @@ struct Registrar* registrarNew(const RegistrarIdentifierType  serverID,
       registrar->PeerHeartbeatCycle                    = REGISTRAR_DEFAULT_PEER_HEARTBEAT_CYCLE;
       registrar->MentorDiscoveryTimeout                = REGISTRAR_DEFAULT_MENTOR_DISCOVERY_TIMEOUT;
       registrar->TakeoverExpiryInterval                = REGISTRAR_DEFAULT_TAKEOVER_EXPIRY_INTERVAL;
+      registrar->AnnounceTTL                           = REGISTRAR_DEFAULT_ANNOUNCE_TTL;
 
-      registrar->StatsFile             = statsFile;
-      registrar->StatsInterval         = statsInterval;
-      registrar->StatsStartTime        = 0;
-      registrar->StatsLine             = 0;
-      registrar->RegistrationCount     = 0;
-      registrar->ReregistrationCount   = 0;
-      registrar->DeregistrationCount   = 0;
-      registrar->HandleResolutionCount = 0;
-      registrar->FailureReportCount    = 0;
-      registrar->SynchronizationCount  = 0;
-      registrar->UpdateCount           = 0;
+      registrar->MaxHRRate                             = REGISTRAR_DEFAULT_MAX_HR_RATE;
+      registrar->MaxEURate                             = REGISTRAR_DEFAULT_MAX_EU_RATE;
+
+#ifdef ENABLE_REGISTRAR_STATISTICS
+      registrar->ActionLogFile                         = actionLogFile;
+      registrar->ActionLogBZFile                       = actionLogBZFile;
+      registrar->StatsFile                             = statsFile;
+      registrar->StatsBZFile                           = statsBZFile;
+      registrar->Stats.StatsInterval                   = statsInterval;
+      registrar->Stats.ActionLogLine                   = 0;
+      registrar->Stats.ActionLogLastActivity           = 0;
+      registrar->Stats.ActionLogStartTime              = getMicroTime();
+      registrar->Stats.StatsStartTime                  = 0;
+      registrar->Stats.StatsLine                       = 0;
+      registrar->Stats.RegistrationCount               = 0;
+      registrar->Stats.ReregistrationCount             = 0;
+      registrar->Stats.DeregistrationCount             = 0;
+      registrar->Stats.HandleResolutionCount           = 0;
+      registrar->Stats.FailureReportCount              = 0;
+      registrar->Stats.SynchronizationCount            = 0;
+      registrar->Stats.HandleUpdateCount               = 0;
+      registrar->Stats.EndpointKeepAliveCount          = 0;
+      registrar->Stats.NeedsWeightedStatValues         = needsWeightedStatValues;
+      initWeightedStatValue(&registrar->Stats.PoolsCount, registrar->Stats.ActionLogStartTime);
+      initWeightedStatValue(&registrar->Stats.PoolElementsCount, registrar->Stats.ActionLogStartTime);
+      initWeightedStatValue(&registrar->Stats.OwnedPoolElementsCount, registrar->Stats.ActionLogStartTime);
+      initWeightedStatValue(&registrar->Stats.PeersCount, registrar->Stats.ActionLogStartTime);
+#endif
 
       autoCloseTimeout = (registrar->AutoCloseTimeout / 1000000);
       if(ext_setsockopt(registrar->ASAPSocket, IPPROTO_SCTP, SCTP_AUTOCLOSE, &autoCloseTimeout, sizeof(autoCloseTimeout)) < 0) {
@@ -180,7 +208,18 @@ struct Registrar* registrarNew(const RegistrarIdentifierType  serverID,
          logerror("setsockopt() for SCTP_AUTOCLOSE failed");
          LOG_END
       }
-#ifdef sctp_sack_info
+      noDelayOn = 1;
+      if(ext_setsockopt(registrar->ASAPSocket, IPPROTO_SCTP, SCTP_NODELAY, &noDelayOn, sizeof(noDelayOn)) < 0) {
+         LOG_ERROR
+         logerror("setsockopt() for SCTP_NODELAY failed");
+         LOG_END
+      }
+      if(ext_setsockopt(registrar->ENRPUnicastSocket, IPPROTO_SCTP, SCTP_NODELAY, &noDelayOn, sizeof(noDelayOn)) < 0) {
+         LOG_ERROR
+         logerror("setsockopt() for SCTP_NODELAY failed");
+         LOG_END
+      }
+#ifdef HAVE_SCTP_DELAYED_SACK
       /* ====== Tune SACK handling ======================================= */
       /* Without this tuning, the PE would wait 200ms to acknowledge a
          Endpoint Unreachable. The usually immediately following
@@ -262,13 +301,16 @@ struct Registrar* registrarNew(const RegistrarIdentifierType  serverID,
       }
 #endif
 
-      if(registrar->StatsFile) {
+#ifdef ENABLE_REGISTRAR_STATISTICS
+      if(registrar->Stats.StatsInterval > 0) {
          timerNew(&registrar->StatsTimer,
                   &registrar->StateMachine,
                   statisticsCallback,
                   (void*)registrar);
          timerStart(&registrar->StatsTimer, 0);
       }
+      registrarBeginActionLog(registrar);
+#endif
    }
 
    return(registrar);
@@ -279,12 +321,15 @@ struct Registrar* registrarNew(const RegistrarIdentifierType  serverID,
 void registrarDelete(struct Registrar* registrar)
 {
    if(registrar) {
+#ifdef ENABLE_REGISTRAR_STATISTICS
       if(registrar->StatsFile) {
          timerDelete(&registrar->StatsTimer);
       }
+#endif
       fdCallbackDelete(&registrar->ENRPUnicastSocketFDCallback);
       fdCallbackDelete(&registrar->ASAPSocketFDCallback);
       ST_CLASS(peerListManagementDelete)(&registrar->Peers);
+      ST_CLASS(poolUserListDelete)(&registrar->PoolUsers);
       ST_CLASS(poolHandlespaceManagementDelete)(&registrar->Handlespace);
 #ifdef ENABLE_CSP
       if(registrar->CSPReportInterval > 0) {
@@ -312,6 +357,10 @@ void registrarDelete(struct Registrar* registrar)
          ext_close(registrar->ASAPAnnounceSocket);
          registrar->ASAPAnnounceSocket = -1;
       }
+      if(registrar->ASAPSocket >= 0) {
+         ext_close(registrar->ASAPSocket);
+         registrar->ASAPSocket = -1;
+      }
       dispatcherDelete(&registrar->StateMachine);
       messageBufferDelete(registrar->ENRPUnicastMessageBuffer);
       registrar->ENRPUnicastMessageBuffer = NULL;
@@ -344,6 +393,7 @@ static void peerListNodeDisposer(struct ST_CLASS(PeerListNode)* peerListNode,
 }
 
 
+#ifdef ENABLE_REGISTRAR_STATISTICS
 #ifdef LINUX
 /* ###### Get system uptime (in microseconds) ############################ */
 static unsigned long long getUptime()
@@ -426,16 +476,28 @@ static void statisticsCallback(struct Dispatcher* dispatcher,
    unsigned long long       runtime     = 0;
    unsigned long long       userTime    = 0;
    unsigned long long       systemTime  = 0;
+   int                      bzerror;
+   const char*              header;
+   char                     str[1024];
 #ifdef LINUX
    unsigned long long       startupTime = 0;
    unsigned long long       uptime;
 #endif
+   size_t                   peers;
+   size_t                   pools;
+   size_t                   poolElements;
+   size_t                   ownedPoolElements;
 
-   if(registrar->StatsLine == 0) {
-      fputs("AbsTime RelTime Runtime UserTime SystemTime Registrations Reregistrations Deregistrations HandleResolutions FailureReports Synchronizations Updates\n",
-            registrar->StatsFile);
-      registrar->StatsLine      = 1;
-      registrar->StatsStartTime = now;
+   if(registrar->Stats.StatsLine == 0) {
+      header = "AbsTime RelTime Runtime UserTime SystemTime   Peers Pools PoolElements OwnedPoolElements   Registrations Reregistrations Deregistrations HandleResolutions FailureReports Synchronizations HandleUpdates EndpointKeepAlives\n";
+      if(registrar->StatsBZFile) {
+         BZ2_bzWrite(&bzerror, registrar->StatsBZFile, (char*)header, strlen(header));
+      }
+      else {
+         fputs(header, registrar->StatsFile);
+      }
+      registrar->Stats.StatsLine      = 1;
+      registrar->Stats.StatsStartTime = now;
    }
 
 #ifdef LINUX
@@ -451,29 +513,145 @@ static void statisticsCallback(struct Dispatcher* dispatcher,
    }
 #endif
 
-   fprintf(registrar->StatsFile,
-           "%06llu %1.6f %1.6f   %1.6f %1.6f %1.6f   %llu %llu %llu %llu %llu %llu %llu\n",
-           registrar->StatsLine++,
-           now / 1000000.0,
-           (now - registrar->StatsStartTime) / 1000000.0,
+   peers             = ST_CLASS(peerListManagementGetPeers)(&registrar->Peers);
+   pools             = ST_CLASS(poolHandlespaceManagementGetPools)(&registrar->Handlespace);
+   poolElements      = ST_CLASS(poolHandlespaceManagementGetPoolElements)(&registrar->Handlespace);
+   ownedPoolElements = ST_CLASS(poolHandlespaceManagementGetOwnedPoolElements)(&registrar->Handlespace);
 
-           runtime / 1000000.0,
-           userTime / 1000000.0,
-           systemTime / 1000000.0,
+   snprintf((char*)&str, sizeof(str),
+            "%06llu %1.6f %1.6f   %1.6f %1.6f %1.6f   %llu %llu %llu %llu   %llu %llu %llu %llu %llu %llu %llu %llu\n",
+            registrar->Stats.StatsLine++,
+            now / 1000000.0,
+            (now - registrar->Stats.StatsStartTime) / 1000000.0,
 
-           registrar->RegistrationCount,
-           registrar->ReregistrationCount,
-           registrar->DeregistrationCount,
-           registrar->HandleResolutionCount,
-           registrar->FailureReportCount,
-           registrar->SynchronizationCount,
-           registrar->UpdateCount);
-   fflush(registrar->StatsFile);
+            runtime / 1000000.0,
+            userTime / 1000000.0,
+            systemTime / 1000000.0,
 
-   timerStart(timer, getMicroTime() + (1000ULL * registrar->StatsInterval));
+            (unsigned long long)peers,
+            (unsigned long long)pools,
+            (unsigned long long)poolElements,
+            (unsigned long long)ownedPoolElements,
+
+            registrar->Stats.RegistrationCount,
+            registrar->Stats.ReregistrationCount,
+            registrar->Stats.DeregistrationCount,
+            registrar->Stats.HandleResolutionCount,
+            registrar->Stats.FailureReportCount,
+            registrar->Stats.SynchronizationCount,
+            registrar->Stats.HandleUpdateCount,
+            registrar->Stats.EndpointKeepAliveCount);
+   if(registrar->StatsBZFile) {
+      BZ2_bzWrite(&bzerror, registrar->StatsBZFile, str, strlen(str));
+   }
+   else {
+      fputs(str, registrar->StatsFile);
+      fflush(registrar->StatsFile);
+   }
+
+   timerStart(timer, getMicroTime() + (1000ULL * registrar->Stats.StatsInterval));
 }
 
 
+/* ###### Print scalar statistics ######################################## */
+void registrarPrintScalarStatistics(struct Registrar* registrar,
+                                    FILE*             fh,
+                                    const char*       objectName)
+{
+   const unsigned long long now = getMicroTime();
+
+   fputs("run 1 \"scenario\"\n", fh);
+   fprintf(fh, "scalar \"%s\" \"Registrar Total Registrations\"        %8llu\n", objectName, registrar->Stats.RegistrationCount);
+   fprintf(fh, "scalar \"%s\" \"Registrar Total Reregistrations\"      %8llu\n", objectName, registrar->Stats.ReregistrationCount);
+   fprintf(fh, "scalar \"%s\" \"Registrar Total Deregistrations\"      %8llu\n", objectName, registrar->Stats.DeregistrationCount);
+   fprintf(fh, "scalar \"%s\" \"Registrar Total Handle Resolutions\"   %8llu\n", objectName, registrar->Stats.HandleResolutionCount);
+   fprintf(fh, "scalar \"%s\" \"Registrar Total Failure Reports\"      %8llu\n", objectName, registrar->Stats.FailureReportCount);
+   fprintf(fh, "scalar \"%s\" \"Registrar Total Synchronizations\"     %8llu\n", objectName, registrar->Stats.SynchronizationCount);
+   fprintf(fh, "scalar \"%s\" \"Registrar Total Handle Updates\"       %8llu\n", objectName, registrar->Stats.HandleUpdateCount);
+   fprintf(fh, "scalar \"%s\" \"Registrar Total Endpoint Keep Alives\" %8llu\n", objectName, registrar->Stats.EndpointKeepAliveCount);
+
+   fprintf(fh, "scalar \"%s\" \"Registrar Average Number Of Pools\"               %1.6f\n", objectName, averageWeightedStatValue(&registrar->Stats.PoolsCount, now));
+   fprintf(fh, "scalar \"%s\" \"Registrar Average Number Of Pool Elements\"       %1.6f\n", objectName, averageWeightedStatValue(&registrar->Stats.PoolElementsCount, now));
+   fprintf(fh, "scalar \"%s\" \"Registrar Average Number Of Owned Pool Elements\" %1.6f\n", objectName, averageWeightedStatValue(&registrar->Stats.OwnedPoolElementsCount, now));
+   fprintf(fh, "scalar \"%s\" \"Registrar Average Number Of Peers\"               %1.6f\n", objectName, averageWeightedStatValue(&registrar->Stats.PeersCount, now));
+}
+
+
+/* ###### Write action log header line ################################### */
+void registrarBeginActionLog(struct Registrar* registrar)
+{
+   const char* header;
+   int         bzerror;
+
+   if(registrar->ActionLogFile) {
+      header = "AbsTime RelTime   Direction Protocol Action Reason   Flags Counter Time   PoolHandle PoolElementID   SenderID ReceiverID TargetID   ErrorCode\n";
+      if(registrar->ActionLogBZFile) {
+         BZ2_bzWrite(&bzerror, registrar->ActionLogBZFile, (char*)header, strlen(header));
+      }
+      else {
+         fputs(header, registrar->ActionLogFile);
+         fflush(registrar->ActionLogFile);
+      }
+   }
+}
+
+
+/* ###### Write action log entry ######################################### */
+void registrarWriteActionLog(struct Registrar*         registrar,
+                             const char*               direction,
+                             const char*               protocol,
+                             const char*               action,
+                             const char*               reason,
+                             const uint32_t            flags,
+                             const unsigned long long  counter,
+                             const unsigned long long  timeValue,
+                             const struct PoolHandle*  poolHandle,
+                             PoolElementIdentifierType poolElementID,
+                             RegistrarIdentifierType   senderID,
+                             RegistrarIdentifierType   receiverID,
+                             RegistrarIdentifierType   targetID,
+                             unsigned int              errorCode)
+{
+   char str1[1024];
+   char str2[1024];
+   char str3[256];
+   int  bzerror;
+
+   if(registrar->ActionLogFile) {
+      registrar->Stats.ActionLogLastActivity = getMicroTime();
+      registrar->Stats.ActionLogLine++;
+      snprintf((char*)&str1, sizeof(str1),
+               "%06llu   %1.6f %1.6f   \"%s\" \"%s\" \"%s\" \"%s\"   0x%x %llu %1.6f   \"",
+               registrar->Stats.ActionLogLine,
+               registrar->Stats.ActionLogLastActivity / 1000000.0,
+               (registrar->Stats.ActionLogLastActivity - registrar->Stats.ActionLogStartTime) / 1000000.0,
+               direction, protocol, action, reason, flags, counter, timeValue / 1000000.0);
+      if(poolHandle) {
+         poolHandleGetDescription(poolHandle, (char*)&str2, sizeof(str2));
+      }
+      else {
+         str2[0] = 0x00;
+      }
+      snprintf((char*)&str3, sizeof(str3),
+               "\" 0x%x   0x%x 0x%x 0x%x   %x\n",
+               poolElementID, senderID, receiverID, targetID, errorCode);
+
+      if(registrar->ActionLogBZFile) {
+         BZ2_bzWrite(&bzerror, registrar->ActionLogBZFile, str1, strlen(str1));
+         BZ2_bzWrite(&bzerror, registrar->ActionLogBZFile, str2, strlen(str2));
+         BZ2_bzWrite(&bzerror, registrar->ActionLogBZFile, str3, strlen(str3));
+      }
+      else {
+         fputs(str1, registrar->ActionLogFile);
+         fputs(str2, registrar->ActionLogFile);
+         fputs(str3, registrar->ActionLogFile);
+         fflush(registrar->ActionLogFile);
+      }
+   }
+}
+#endif
+
+
 #ifdef ENABLE_CSP
 /* ###### Get CSP report ################################################# */
 static size_t registrarGetReportFunction(
