@@ -1,4 +1,4 @@
-/* $Id: registrar.c 1732 2008-02-14 12:14:23Z dreibh $
+/* $Id: registrar.c 2076 2008-12-28 15:52:34Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -9,7 +9,7 @@
  *
  * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
- * Copyright (C) 2002-2008 by Thomas Dreibholz
+ * Copyright (C) 2002-2009 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,6 +29,13 @@
 
 #include "registrar.h"
 
+#include <signal.h>
+
+
+/*
+#define FAST_BREAK
+*/
+
 
 /* ###### Add peer ####################################################### */
 static void addPeer(struct Registrar* registrar, char* arg)
@@ -250,48 +257,71 @@ static void getSocketPair(const char*                   sctpAddressParameter,
 }
 
 
+/* ###### Enter daemon mode, if requested ################################ */
+static void goIntoDaemonMode(const char* daemonPIDFile)
+{
+   FILE* fh;
+   pid_t childProcess;
+
+   if(daemonPIDFile != NULL) {
+      childProcess = fork();
+      if(childProcess != 0) {
+         fh = fopen(daemonPIDFile, "w");
+         if(fh) {
+            fprintf(fh, "%d\n", childProcess);
+            fclose(fh);
+         }
+         else {
+            kill(childProcess, SIGKILL);
+            fprintf(stderr, "ERROR: Unable to create PID file \"%s\"!\n", daemonPIDFile);
+         }
+         exit(0);
+      }
+   }
+}
+
+
 
 /* ###### Main program ################################################### */
 int main(int argc, char** argv)
 {
-   struct Registrar*             registrar;
-   uint32_t                      serverID                      = 0;
-   bool                          quiet                         = false;
-
    char                          asapUnicastAddressBuffer[transportAddressBlockGetSize(MAX_PE_TRANSPORTADDRESSES)];
    struct TransportAddressBlock* asapUnicastAddress            = (struct TransportAddressBlock*)&asapUnicastAddressBuffer;
-   const char*                   asapUnicastAddressParameter   = "auto";
-   int                           asapUnicastSocket             = -1;
+   const char*                   asapUnicastAddressParameter;
+   int                           asapUnicastSocket;
 
    char                          asapAnnounceAddressBuffer[transportAddressBlockGetSize(1)];
    struct TransportAddressBlock* asapAnnounceAddress           = (struct TransportAddressBlock*)&asapAnnounceAddressBuffer;
-   const char*                   asapAnnounceAddressParameter  = "auto";
-   int                           asapAnnounceSocket            = -1;
-   bool                          asapSendAnnounces             = false;
+   const char*                   asapAnnounceAddressParameter;
+   int                           asapAnnounceSocket;
+   bool                          asapSendAnnounces;
 
    char                          enrpUnicastAddressBuffer[transportAddressBlockGetSize(MAX_PE_TRANSPORTADDRESSES)];
    struct TransportAddressBlock* enrpUnicastAddress            = (struct TransportAddressBlock*)&enrpUnicastAddressBuffer;
-   const char*                   enrpUnicastAddressParameter   = "auto";
-   int                           enrpUnicastSocket             = -1;
+   const char*                   enrpUnicastAddressParameter;
+   int                           enrpUnicastSocket;
 
    char                          enrpMulticastAddressBuffer[transportAddressBlockGetSize(1)];
-   struct TransportAddressBlock* enrpMulticastAddress          = (struct TransportAddressBlock*)&enrpMulticastAddressBuffer;
-   const char*                   enrpMulticastAddressParameter = "auto";
-   int                           enrpMulticastOutputSocket     = -1;
-   int                           enrpMulticastInputSocket      = -1;
-   bool                          enrpAnnounceViaMulticast      = false;
-   bool                          useIPv6                       = checkIPv6();
-   const char*                   daemonPIDFile                 = NULL;
-   FILE*                         fh;
-   pid_t                         childProcess;
+   struct TransportAddressBlock* enrpMulticastAddress = (struct TransportAddressBlock*)&enrpMulticastAddressBuffer;
+   const char*                   enrpMulticastAddressParameter;
+   int                           enrpMulticastOutputSocket;
+   int                           enrpMulticastInputSocket;
+   bool                          enrpAnnounceViaMulticast;
 
-#ifdef ENABLE_CSP
-   union sockaddr_union          cspReportAddress;
-   unsigned long long            cspReportInterval = 0;
-#endif
+   struct Registrar*             registrar;
+   struct RegistrarStatistics    statistics;
+   uint32_t                      serverID ;
+   bool                          quiet;
 
-   FILE*                         statsFile     = NULL;
-   int                           statsInterval = -1;
+   bool                          useIPv6;
+   const char*                   daemonPIDFile;
+
+   unsigned int                  run;
+   double                        uptime;
+   double                        downtime;
+   double                        t;
+   unsigned long long            endTimeStamp;
+   unsigned long long            now;
 
    unsigned long long            pollTimeStamp;
    struct pollfd                 ufds[FD_SETSIZE];
@@ -300,6 +330,42 @@ int main(int argc, char** argv)
    int                           result;
    int                           i;
 
+#ifdef ENABLE_CSP
+   union sockaddr_union          cspReportAddress;
+   unsigned long long            cspReportInterval = 0;
+#endif
+
+#ifdef ENABLE_REGISTRAR_STATISTICS
+   const char*                   objectName      = "registrar";
+   const char*                   scalarName      = NULL;
+   FILE*                         scalarFH        = NULL;
+   FILE*                         actionLogFile   = NULL;
+   BZFILE*                       actionLogBZFile = NULL;
+   FILE*                         statsFile       = NULL;
+   BZFILE*                       statsBZFile     = NULL;
+   int                           statsInterval   = -1;
+   int                           bzerror;
+#endif
+
+   run                           = 1;
+   uptime                        = 0.0;
+   downtime                      = 0.0;
+start:
+   serverID                      = 0;
+   quiet                         = false;
+   useIPv6                       = checkIPv6();
+   daemonPIDFile                 = NULL;
+   asapUnicastAddressParameter   = "auto";
+   asapUnicastSocket             = -1;
+   asapAnnounceAddressParameter  = "auto";
+   asapAnnounceSocket            = -1;
+   asapSendAnnounces             = false;
+   enrpUnicastAddressParameter   = "auto";
+   enrpUnicastSocket             = -1;
+   enrpMulticastAddressParameter = "auto";
+   enrpMulticastOutputSocket     = -1;
+   enrpMulticastInputSocket      = -1;
+   enrpAnnounceViaMulticast      = false;
 
    /* ====== Get arguments =============================================== */
 #ifdef ENABLE_CSP
@@ -337,8 +403,11 @@ int main(int argc, char** argv)
                (!(strncmp(argv[i], "-peermaxtimenoresponse=", 23))) ||
                (!(strncmp(argv[i], "-mentordiscoverytimeout=", 19))) ||
                (!(strncmp(argv[i], "-takeoverexpiryinterval=", 24))) ||
+               (!(strcmp(argv[i], "-supporttakeoversuggestion"))) ||
                (!(strncmp(argv[i], "-maxincrement=", 14))) ||
                (!(strncmp(argv[i], "-maxhresitems=", 14))) ||
+               (!(strncmp(argv[i], "-maxhrrate=", 11))) ||
+               (!(strncmp(argv[i], "-maxeurate=", 11))) ||
                (!(strncmp(argv[i], "-maxelementsperhtrequest=", 25))) ) {
          /* to be handled later */
       }
@@ -382,6 +451,11 @@ int main(int argc, char** argv)
             enrpAnnounceViaMulticast      = true;
          }
       }
+      else if(!(strncmp(argv[i], "-log",4))) {
+         if(initLogging(argv[i]) == false) {
+            exit(1);
+         }
+      }
       else if(!(strcmp(argv[i], "-quiet"))) {
          quiet = true;
       }
@@ -391,15 +465,56 @@ int main(int argc, char** argv)
       else if(!(strncmp(argv[i], "-daemonpidfile=", 15))) {
          daemonPIDFile = (const char*)&argv[i][15];
       }
+      else if(!(strncmp(argv[i], "-uptime=", 8))) {
+         uptime = atof((const char*)&argv[i][8]);
+      }
+      else if(!(strncmp(argv[i], "-downtime=", 10))) {
+         downtime = atof((const char*)&argv[i][10]);
+      }
+#ifdef ENABLE_REGISTRAR_STATISTICS
+      else if(!(strncmp(argv[i], "-actionlogfile=", 15))) {
+         if(actionLogBZFile) {
+            BZ2_bzWriteClose(&bzerror, actionLogBZFile, 0, NULL, NULL);
+            actionLogBZFile = NULL;
+         }
+         if(actionLogFile) {
+            fclose(actionLogFile);
+         }
+         actionLogFile = fopen((const char*)&argv[i][15], (run > 1) ? "a" : "w");
+         if(actionLogFile == NULL) {
+            fprintf(stderr, "ERROR: Unable to create action log file \"%s\"!\n",
+                    (char*)&argv[i][11]);
+         }
+         if(strstr(&argv[i][15], ".bz2") == &argv[i][15 + strlen(&argv[i][15]) - 4]) {
+            actionLogBZFile = BZ2_bzWriteOpen(&bzerror, actionLogFile, 9, 0, 30);
+            if(actionLogBZFile == NULL) {
+               fprintf(stderr, "ERROR: Unable to initialize BZip2 compressor for action log file \"%s\"!\n",
+                       (char*)&argv[i][11]);
+            }
+         }
+      }
+      else if(!(strncmp(argv[i], "-object=", 8))) {
+         objectName = (const char*)&argv[i][8];
+      }
+      else if(!(strncmp(argv[i], "-scalar=", 8))) {
+         scalarName = (const char*)&argv[i][8];
+      }
       else if(!(strncmp(argv[i], "-statsfile=", 11))) {
          if(statsFile) {
             fclose(statsFile);
          }
-         statsFile = fopen((const char*)&argv[i][11], "w");
+         statsFile = fopen((const char*)&argv[i][11], (run > 1) ? "a" : "w");
          if(statsFile == NULL) {
             fprintf(stderr, "ERROR: Unable to create statistics file \"%s\"!\n",
                     (char*)&argv[i][11]);
          }
+         if(strstr(&argv[i][11], ".bz2") == &argv[i][11 + strlen(&argv[i][11]) - 4]) {
+            statsBZFile = BZ2_bzWriteOpen(&bzerror, statsFile, 9, 0, 30);
+            if(statsBZFile == NULL) {
+               fprintf(stderr, "ERROR: Unable to initialize BZip2 compressor for statistics file \"%s\"!\n",
+                       (char*)&argv[i][11]);
+            }
+         }
       }
       else if(!(strncmp(argv[i], "-statsinterval=", 15))) {
          statsInterval = atol((const char*)&argv[i][15]);
@@ -410,11 +525,7 @@ int main(int argc, char** argv)
             statsInterval = 36000000;
          }
       }
-      else if(!(strncmp(argv[i], "-log",4))) {
-         if(initLogging(argv[i]) == false) {
-            exit(1);
-         }
-      }
+#endif
 #ifdef ENABLE_CSP
       else if(!(strncmp(argv[i], "-cspinterval=", 13))) {
          cspReportInterval = 1000ULL * atol((char*)&argv[i][13]);
@@ -444,24 +555,51 @@ int main(int argc, char** argv)
          fprintf(stderr, "ERROR: Invalid argument <%s>!\n", argv[i]);
          fprintf(stderr, "Usage: %s {-asap=auto|address:port{,address}...} {[-asapannounce=auto|address:port}]} {-enrp=auto|address:port{,address}...} {[-enrpannounce=auto|address:port}]} {-logfile=file|-logappend=file|-logquiet} {-loglevel=level} {-logcolor=on|off} "
 #ifdef ENABLE_CSP
-            "{-cspserver=address} {-cspinterval=microseconds} "
+            "{-cspserver=address} {-cspinterval=milliseconds} "
 #endif
             "{-identifier=registrar identifier} "
             "{-disable-ipv6} {-quiet} "
             "{-autoclosetimeout=seconds} {-serverannouncecycle=milliseconds} "
-            "{-maxbadpereports=reports} "
+            "{-maxbadpereports=reports} {-maxeurate=rate} {-maxhrrate=rate} "
             "{-endpointkeepalivetransmissioninterval=milliseconds} {-endpointkeepalivetimeoutinterval=milliseconds} "
             "{-minaddressscope=loopback|sitelocal|global} "
             "{-peerheartbeatcycle=milliseconds} {-peermaxtimelastheard=milliseconds} {-peermaxtimenoresponse=milliseconds} "
-            "{-takeoverexpiryinterval=milliseconds} {-mentorhuntinterval=milliseconds} "
-            "{-statsfile=file} {-statsinterval=millisecs} "
+            "{-supporttakeoversuggestion} {-takeoverexpiryinterval=milliseconds} {-mentorhuntinterval=milliseconds} "
+#ifdef ENABLE_REGISTRAR_STATISTICS
+            "{-actionlogfile=file} {-statsfile=file} {-statsinterval=millisecs} {-scalar=file} {-object=ID} "
+#endif
             "{-daemonpidfile=file}"
             "\n",argv[0]);
          exit(1);
       }
    }
+
+   /* ====== Failure test mode =========================================== */
+   if(uptime < 0.000001) {
+      uptime = 0.0;
+   }
+   if(downtime < 0.000001) {
+      downtime = 0.000001;
+   }
+   if(uptime > 0.000001) {
+      printf("Uptime   = %1.3f [s]\n", uptime);
+      printf("Downtime = %1.3f [s]\n", downtime);
+      t = randomExpDouble(uptime);
+      printf("\nRun #%u: Running registrar for %1.3fs ...\n\n", run, t);
+      endTimeStamp = getMicroTime() + (unsigned long long)rint(t * 1000000.0);
+   }
+   else {
+      endTimeStamp = (uptime > 0) ? (getMicroTime() + uptime) : 0;
+   }
+
    beginLogging();
 
+#ifndef HAVE_KERNEL_SCTP
+   /* For userland SCTP: go into daemon mode now. This mode must be entered
+      before any SCTP is used (fork() limitation for userland SCTP)! */
+   goIntoDaemonMode(daemonPIDFile);
+#endif
+
    if(!strcmp(asapAnnounceAddressParameter, "auto")) {
       asapAnnounceAddressParameter = ASAP_ANNOUNCE_MULTICAST_ADDRESS;
       asapSendAnnounces            = true;
@@ -504,8 +642,10 @@ int main(int argc, char** argv)
                             enrpMulticastInputSocket,
                             enrpMulticastOutputSocket,
                             asapSendAnnounces, (const union sockaddr_union*)&asapAnnounceAddress->AddressArray[0],
-                            enrpAnnounceViaMulticast, (const union sockaddr_union*)&enrpMulticastAddress->AddressArray[0],
-                            statsFile, statsInterval
+                            enrpAnnounceViaMulticast, (const union sockaddr_union*)&enrpMulticastAddress->AddressArray[0]
+#ifdef ENABLE_REGISTRAR_STATISTICS
+                            , actionLogFile, actionLogBZFile, statsFile, statsBZFile, statsInterval, (scalarName != NULL)
+#endif
 #ifdef ENABLE_CSP
                             , cspReportInterval, &cspReportAddress
 #endif
@@ -514,6 +654,11 @@ int main(int argc, char** argv)
       fprintf(stderr, "ERROR: Unable to initialize Registrar object!\n");
       exit(1);
    }
+#ifdef ENABLE_REGISTRAR_STATISTICS
+   if(run > 1) {
+      registrar->Stats = statistics;
+   }
+#endif
    for(i = 1;i < argc;i++) {
       if(!(strncmp(argv[i], "-peer=",6))) {
          addPeer(registrar, (char*)&argv[i][6]);
@@ -638,6 +783,24 @@ int main(int argc, char** argv)
             registrar->TakeoverExpiryInterval = 60000000;
          }
       }
+      else if(!(strncmp(argv[i], "-announcettl=", 13))) {
+         registrar->AnnounceTTL = atol((char*)&argv[i][13]);
+         if(registrar->AnnounceTTL < 1) {
+            registrar->AnnounceTTL = 1;
+         }
+         else if(registrar->AnnounceTTL > 255) {
+            registrar->AnnounceTTL = 255;
+         }
+      }
+      else if(!(strncmp(argv[i], "-maxhrrate=", 11))) {
+         registrar->MaxHRRate = atof((const char*)&argv[i][11]);
+      }
+      else if(!(strncmp(argv[i], "-maxeurate=", 11))) {
+         registrar->MaxEURate = atof((const char*)&argv[i][11]);
+      }
+      else if(!(strcmp(argv[i], "-supporttakeoversuggestion"))) {
+         registrar->ENRPSupportTakeoverSuggestion = true;
+      }
    }
 #ifndef FAST_BREAK
    installBreakDetector();
@@ -670,14 +833,15 @@ int main(int argc, char** argv)
       else {
          puts("(none)");
       }
-   #ifdef ENABLE_CSP
+      printf("ASAP/ENRP Announce TTL: %d\n", registrar->AnnounceTTL);
+#ifdef ENABLE_CSP
       if(cspReportInterval > 0) {
          printf("CSP Report Address:     ");
          fputaddress((struct sockaddr*)&cspReportAddress, true, stdout);
          puts("");
          printf("CSP Report Interval:    %lldms\n", cspReportInterval / 1000);
       }
-   #endif
+#endif
       printf("Auto-Close Timeout:     %llus\n", registrar->AutoCloseTimeout / 1000000);
       printf("Min Address Scope:      ");
       if(registrar->MinEndpointAddressScope <= AS_LOOPBACK) {
@@ -689,9 +853,18 @@ int main(int argc, char** argv)
       else {
          puts("global");
       }
+#ifdef ENABLE_REGISTRAR_STATISTICS
       if(statsFile) {
          printf("Statistics Interval:    %ums\n", statsInterval);
       }
+      if(actionLogFile) {
+         printf("Action Log File:        active\n");
+      }
+      if(scalarName) {
+         printf("Scalar File:            %s\n", scalarName);
+         printf("Object ID:              %s\n", objectName);
+      }
+#endif
       printf("Daemon Mode:            %s\n", (daemonPIDFile == NULL) ? "off" : daemonPIDFile);
 
       puts("\nASAP Parameters:");
@@ -710,37 +883,40 @@ int main(int argc, char** argv)
       printf("   Max Elements per Handle Table Request:       %u\n",     (unsigned int)registrar->MaxElementsPerHTRequest);
       printf("   Mentor Hunt Timeout:                         %lldms\n", registrar->MentorDiscoveryTimeout / 1000);
       printf("   Takeover Expiry Interval:                    %lldms\n", registrar->TakeoverExpiryInterval / 1000);
+      printf("   Support for Takeover Suggestion:             %s\n", registrar->ENRPSupportTakeoverSuggestion ? "on" : "off");
+      puts("Security Parameters:");
+      printf("   Max Handle Resolution Rate:                  ");
+      if(registrar->MaxHRRate > 0.0) {
+         printf("%1.2lf requests/s\n", registrar->MaxHRRate);
+      }
+      else {
+         puts("unlimited");
+      }
+      printf("   Max Endpoint Unreachable Rate:               ");
+      if(registrar->MaxEURate > 0.0) {
+         printf("%1.2lf requests/s\n", registrar->MaxEURate);
+      }
+      else {
+         puts("unlimited");
+      }
       puts("");
    }
 
-
    /* ====== We are ready! =============================================== */
    LOG_NOTE
    fputs("Registrar started. Going into initialization phase...\n", stdlog);
    LOG_END
 
-
-   if(daemonPIDFile != NULL) {
-      childProcess = fork();
-      if(childProcess != 0) {
-         fh = fopen(daemonPIDFile, "w");
-         if(fh) {
-            fprintf(fh, "%d\n", childProcess);
-            fclose(fh);
-         }
-         else {
-            fprintf(stderr, "ERROR: Unable to create PID file \"%s\"!\n", daemonPIDFile);
-         }
-         exit(0);
-      }
-   }
+#ifdef HAVE_KERNEL_SCTP
+   goIntoDaemonMode(daemonPIDFile);
+#endif
 
    /* ====== Main loop =================================================== */
    while(!breakDetected()) {
       dispatcherGetPollParameters(&registrar->StateMachine,
                                   (struct pollfd*)&ufds, &nfds, &timeout,
                                   &pollTimeStamp);
-      if((timeout < 0) || (timeout > 250)) {
+      if((timeout < 0) || (timeout > 500)) {
          timeout = 500;
       }
       result = ext_poll((struct pollfd*)&ufds, nfds, timeout);
@@ -750,17 +926,67 @@ int main(int argc, char** argv)
          }
          break;
       }
+      if( (endTimeStamp > 0) && (endTimeStamp <= getMicroTime()) ) {
+         puts("Shutdown by timer!");
+         break;
+      }
       dispatcherHandlePollResult(&registrar->StateMachine, result,
                                  (struct pollfd*)&ufds, nfds, timeout,
                                  pollTimeStamp);
    }
 
    /* ====== Clean up ==================================================== */
-   registrarDelete(registrar);
-   finishLogging();
+#ifdef ENABLE_REGISTRAR_STATISTICS
+   if(registrar->Stats.NeedsWeightedStatValues) {
+      now = getMicroTime();
+      updateWeightedStatValue(&registrar->Stats.PoolsCount, now, 0);
+      updateWeightedStatValue(&registrar->Stats.PoolElementsCount, now, 0);
+      updateWeightedStatValue(&registrar->Stats.OwnedPoolElementsCount, now, 0);
+      updateWeightedStatValue(&registrar->Stats.PeersCount, now, 0);
+   }
+   statistics = registrar->Stats;
+   if(scalarName) {
+       scalarFH = fopen(scalarName, "w");
+       if(scalarFH) {
+          registrarPrintScalarStatistics(registrar, scalarFH, objectName);
+          fclose(scalarFH);
+       }
+       else {
+          fprintf(stderr, "ERROR: Unable to create scalar file \"%s\"!\n", scalarName);
+       }
+   }
+   if(statsBZFile) {
+      BZ2_bzWriteClose(&bzerror, statsBZFile, 0, NULL, NULL);
+      statsBZFile = NULL;
+   }
    if(statsFile) {
       fclose(statsFile);
+      statsFile = NULL;
    }
+   if(actionLogBZFile) {
+      BZ2_bzWriteClose(&bzerror, actionLogBZFile, 0, NULL, NULL);
+      actionLogBZFile = NULL;
+   }
+   if(actionLogFile) {
+      fclose(actionLogFile);
+   }
+#endif
+   registrarDelete(registrar);
+   finishLogging();
+#ifndef FAST_BREAK
+    uninstallBreakDetector();
+#endif
+
+   if((!breakDetected()) && (uptime > 0.000001)) {
+      t = randomExpDouble(downtime);
+      printf("\nWaiting for %1.3fs ...\n", t);
+      usleep((unsigned long long)rint(t * 1000000.0));
+      if(!breakDetected()) {
+         run++;
+         goto start;
+      }
+   }
+
    if(!quiet) {
       puts("\nTerminated!");
    }
