@@ -1,4 +1,4 @@
-/* $Id: rserpool.h 2076 2008-12-28 15:52:34Z dreibh $
+/* $Id: rserpool.h 2112 2009-03-14 12:01:35Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -35,9 +35,9 @@
 #include <netdb.h>
 #include <sys/time.h>
 #include <sys/poll.h>
-#include <ext_socket.h>
 
 #include "rserpool-policytypes.h"
+#include "rserpool-csp.h"
 
 
 #ifdef __cplusplus
