@@ -1,4 +1,4 @@
-/* $Id: rspbasicmode.c 1741 2008-02-14 17:12:54Z dreibh $
+/* $Id: rspbasicmode.c 2076 2008-12-28 15:52:34Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -9,7 +9,7 @@
  *
  * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
- * Copyright (C) 2002-2008 by Thomas Dreibholz
+ * Copyright (C) 2002-2009 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -118,22 +118,6 @@ int rsp_initialize(struct rsp_info* info)
          info->ri_build_time = buildTime;
       }
 
-#ifdef ENABLE_CSP
-      /* ====== Initialize Component Status Reporter ===================== */
-      if((info) &&
-         (info->ri_csp_interval > 0) && (info->ri_csp_server != NULL)) {
-         gCSPReporter = (struct CSPReporter*)malloc(sizeof(struct CSPReporter));
-         if(gCSPReporter) {
-            cspReporterNew(gCSPReporter, &gDispatcher,
-                           info->ri_csp_identifier,
-                           info->ri_csp_server,
-                           1000UL * info->ri_csp_interval,
-                           getComponentStatus,
-                           NULL);
-         }
-      }
-#endif
-
       /* ====== Initialize session storage =============================== */
       simpleRedBlackTreeNew(&gRSerPoolSocketSet,
                             rserpoolSocketPrint,
@@ -152,6 +136,23 @@ int rsp_initialize(struct rsp_info* info)
          /* ====== Add static registrars ================================= */
          if( (info == NULL) ||
              (addStaticRegistrars(gAsapInstance->RegistrarSet, info)) ) {
+
+#ifdef ENABLE_CSP
+            /* ====== Initialize Component Status Reporter =============== */
+            if((info) &&
+               (info->ri_csp_interval > 0) && (info->ri_csp_server != NULL)) {
+               gCSPReporter = (struct CSPReporter*)malloc(sizeof(struct CSPReporter));
+               if(gCSPReporter) {
+                  cspReporterNew(gCSPReporter, &gDispatcher,
+                                 info->ri_csp_identifier,
+                                 info->ri_csp_server,
+                                 1000UL * info->ri_csp_interval,
+                                 getComponentStatus,
+                                 NULL);
+               }
+            }
+#endif
+
             LOG_NOTE
             fputs("rsplib is ready\n", stdlog);
             LOG_END
@@ -431,13 +432,13 @@ void rsp_freeaddrinfo(struct rsp_addrinfo* rspAddrInfo)
 
 
 /* ###### Register pool element ########################################## */
-unsigned int rsp_pe_registration_tags(const unsigned char* poolHandle,
-                                      const size_t         poolHandleSize,
-                                      struct rsp_addrinfo* rspAddrInfo,
-                                      struct rsp_loadinfo* rspLoadInfo,
-                                      unsigned int         registrationLife,
-                                      int                  flags,
-                                      struct TagItem*      tags)
+unsigned int rsp_pe_registration_tags(const unsigned char*       poolHandle,
+                                      const size_t               poolHandleSize,
+                                      struct rsp_addrinfo*       rspAddrInfo,
+                                      const struct rsp_loadinfo* rspLoadInfo,
+                                      const unsigned int         registrationLife,
+                                      const int                  flags,
+                                      struct TagItem*            tags)
 {
    struct PoolHandle                myPoolHandle;
    struct ST_CLASS(PoolElementNode) myPoolElementNode;
@@ -473,7 +474,7 @@ unsigned int rsp_pe_registration_tags(const unsigned char* poolHandle,
          ST_CLASS(poolElementNodeNew)(
             &myPoolElementNode,
             rspAddrInfo->ai_pe_id,
-            0,
+            gAsapInstance->RegistrarIdentifier,
             registrationLife,
             &myPolicySettings,
             myTransportAddressBlock,
@@ -510,12 +511,12 @@ unsigned int rsp_pe_registration_tags(const unsigned char* poolHandle,
 
 
 /* ###### Register pool element ########################################## */
-unsigned int rsp_pe_registration(const unsigned char* poolHandle,
-                                 const size_t         poolHandleSize,
-                                 struct rsp_addrinfo* rspAddrInfo,
-                                 struct rsp_loadinfo* rspLoadInfo,
-                                 unsigned int         registrationLife,
-                                 int                  flags)
+unsigned int rsp_pe_registration(const unsigned char*       poolHandle,
+                                 const size_t               poolHandleSize,
+                                 struct rsp_addrinfo*       rspAddrInfo,
+                                 const struct rsp_loadinfo* rspLoadInfo,
+                                 const unsigned int         registrationLife,
+                                 const int                  flags)
 {
    return(rsp_pe_registration_tags(poolHandle, poolHandleSize,
                                    rspAddrInfo, rspLoadInfo, registrationLife,
@@ -527,7 +528,7 @@ unsigned int rsp_pe_registration(const unsigned char* poolHandle,
 unsigned int rsp_pe_deregistration_tags(const unsigned char* poolHandle,
                                         const size_t         poolHandleSize,
                                         const uint32_t       identifier,
-                                        int                  flags,
+                                        const int            flags,
                                         struct TagItem*      tags)
 {
    struct PoolHandle myPoolHandle;
@@ -552,7 +553,7 @@ unsigned int rsp_pe_deregistration_tags(const unsigned char* poolHandle,
 unsigned int rsp_pe_deregistration(const unsigned char* poolHandle,
                                    const size_t         poolHandleSize,
                                    const uint32_t       identifier,
-                                   int                  flags)
+                                   const int            flags)
 {
    return(rsp_pe_deregistration_tags(poolHandle, poolHandleSize, identifier,
                                      flags, NULL));
