@@ -1,4 +1,4 @@
-/* $Id: rspenhancedmode.c 1741 2008-02-14 17:12:54Z dreibh $
+/* $Id: rspenhancedmode.c 2076 2008-12-28 15:52:34Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -9,7 +9,7 @@
  *
  * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
- * Copyright (C) 2002-2008 by Thomas Dreibholz
+ * Copyright (C) 2002-2009 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -494,12 +494,8 @@ int rsp_listen(int sd, int backlog)
 {
    struct RSerPoolSocket* rserpoolSocket;
    GET_RSERPOOL_SOCKET(rserpoolSocket, sd);
-
-   if(rserpoolSocket->PoolElement) {
-      return(ext_listen(rserpoolSocket->Socket, backlog));
-   }
-   errno = EBADF;
-   return(-1);
+   /* printf("backlog(%d)=%d\n", sd, backlog); */
+   return(ext_listen(rserpoolSocket->Socket, backlog));
 }
 
 
@@ -569,14 +565,6 @@ int rsp_register_tags(int                        sd,
 
    /* ====== Registration of a new pool element ========================== */
    else {
-      if(ext_listen(rserpoolSocket->Socket, 10) < 0) {
-         LOG_ERROR
-         logerror("Unable to set socket for new pool element to listen mode");
-         LOG_END
-         threadSafetyUnlock(&rserpoolSocket->Mutex);
-         return(-1);
-      }
-
       /* ====== Create pool element ====================================== */
       rserpoolSocket->PoolElement = (struct PoolElement*)malloc(sizeof(struct PoolElement));
       if(rserpoolSocket->PoolElement == NULL) {
@@ -780,7 +768,6 @@ int rsp_accept(int sd,
 
 /* ###### Establish association to a new PE ############################## */
 static int connectToPE(struct RSerPoolSocket* rserpoolSocket,
-                       const uint32_t         peIdentifier,
                        const struct sockaddr* destinationAddressArray,
                        const size_t           destinationAddresses)
 {
@@ -797,10 +784,10 @@ static int connectToPE(struct RSerPoolSocket* rserpoolSocket,
       setNonBlocking(sd);
       if(configureSCTPSocket(sd, 0)) {
          LOG_VERBOSE
-         fprintf(stdlog, "Trying connection to pool element $%08x (", peIdentifier);
+         fprintf(stdlog, "Trying connection to pool element $%08x (", rserpoolSocket->ConnectedSession->ConnectedPE);
          poolHandlePrint(&rserpoolSocket->ConnectedSession->Handle, stdlog);
          fprintf(stdlog, "/$%08x on RSerPool socket %u, socket %d, session %u)\n",
-                 peIdentifier,
+                 rserpoolSocket->ConnectedSession->ConnectedPE,
                  rserpoolSocket->Descriptor, sd,
                  rserpoolSocket->ConnectedSession->SessionID);
          LOG_END
@@ -824,10 +811,10 @@ static int connectToPE(struct RSerPoolSocket* rserpoolSocket,
          }
 
          LOG_VERBOSE
-         fprintf(stdlog, "Failed to establish connection to pool element $%08x (", peIdentifier);
+         fprintf(stdlog, "Failed to establish connection to pool element $%08x (", rserpoolSocket->ConnectedSession->ConnectedPE);
          poolHandlePrint(&rserpoolSocket->ConnectedSession->Handle, stdlog);
          fprintf(stdlog, "/$%08x on RSerPool socket %u, socket %d, session %u)\n",
-                 peIdentifier,
+                 rserpoolSocket->ConnectedSession->ConnectedPE,
                  rserpoolSocket->Descriptor, rserpoolSocket->Socket,
                  rserpoolSocket->ConnectedSession->SessionID);
          LOG_END
@@ -947,14 +934,13 @@ int rsp_forcefailover_tags(int                sd,
       if(result > 0) {
          if(rspAddrInfo->ai_protocol == rserpoolSocket->SocketProtocol) {
             /* ====== Establish connection ================================== */
+            rserpoolSocket->ConnectedSession->ConnectedPE = rspAddrInfo->ai_pe_id;
             rserpoolSocket->Socket = connectToPE(rserpoolSocket,
-                                                 rspAddrInfo->ai_pe_id,
                                                  rspAddrInfo->ai_addr,
                                                  rspAddrInfo->ai_addrs);
             if(rserpoolSocket->Socket >= 0) {
                success = true;
                rserpoolSocket->ConnectedSession->ConnectionTimeStamp = getMicroTime();
-               rserpoolSocket->ConnectedSession->ConnectedPE         = rspAddrInfo->ai_pe_id;
                sessionStorageUpdateSession(&rserpoolSocket->SessionSet,
                                            rserpoolSocket->ConnectedSession,
                                            0);
@@ -1130,16 +1116,18 @@ ssize_t rsp_recvmsg(int                    sd,
                     int*                   msg_flags,
                     int                    timeout)
 {
-   struct RSerPoolSocket* rserpoolSocket;
-   struct Session*        session;
-   struct rsp_sndrcvinfo  rinfoDummy;
-   sctp_assoc_t           assocID;
-   int                    flags;
-   ssize_t                received;
-   ssize_t                received2;
-   unsigned long long     startTimeStamp;
-   unsigned long long     currentTimeout;
-   unsigned long long     now;
+   struct RSerPoolSocket*         rserpoolSocket;
+   struct Session*                session;
+   struct rsp_sndrcvinfo          rinfoDummy;
+   const union sctp_notification* notification;
+   sctp_assoc_t                   assocID;
+   int                            flags;
+   int                            errorCode;
+   ssize_t                        received;
+   ssize_t                        received2;
+   unsigned long long             startTimeStamp;
+   unsigned long long             currentTimeout;
+   unsigned long long             now;
 
    GET_RSERPOOL_SOCKET(rserpoolSocket, sd);
    if(rinfo == NULL) {
@@ -1191,14 +1179,17 @@ ssize_t rsp_recvmsg(int                    sd,
                                       &assocID,
                                       &rinfo->rinfo_stream,
                                       currentTimeout);
+         errorCode = errno;
          LOG_VERBOSE
-         fprintf(stdlog, "received=%d\n", (int)received);
+         fprintf(stdlog, "received=%d errno=%d\n", (int)received, errorCode);
          LOG_END
 
          /* Note, messageBufferRead()'s PPID byte order is host byte order! */
          rinfo->rinfo_ppid = htonl(rinfo->rinfo_ppid);
 
-         if(received == 0) {
+         if( (received == 0) ||
+             ((received < 0) && (errorCode != EAGAIN)) ) {
+            /* A failover is necessary due to error or shutdown. */
             threadSafetyLock(&rserpoolSocket->Mutex);
             if(rserpoolSocket->ConnectedSession) {
                struct NotificationNode* notificationNode;
@@ -1244,9 +1235,17 @@ ssize_t rsp_recvmsg(int                    sd,
 
          /* ====== Handle notification ====================================== */
          if(flags & MSG_NOTIFICATION) {
-            handleNotification(rserpoolSocket,
-                               (const union sctp_notification*)rserpoolSocket->MsgBuffer->Buffer);
-            received = -1;
+            notification = (const union sctp_notification*)rserpoolSocket->MsgBuffer->Buffer;
+            if(handleNotification(rserpoolSocket, notification) == true) {
+               /* The association is closed here. Therefore, we return length 0
+                  to signalize the disconnect. If there is a corresponding
+                  notification, we will copy it later; otherwise, for
+                  SOCK_STREAM, the 0 is returned back. */
+               received = 0;
+            }
+            else {
+               received = -1;
+            }
          }
 
          /* ====== Handle ASAP control channel message ====================== */
@@ -1364,8 +1363,8 @@ ssize_t rsp_recvfullmsg(int                    sd,
    ssize_t                  received;
 
    while( ((received = rsp_recvmsg(sd, (char*)&((char*)buffer)[offset], bufferLength - offset,
-                                  rinfo, msg_flags,
-                                  ((endTimeStamp - now) > 0) ? (int)((endTimeStamp - now) / 1000) : 0)) > 0) &&
+                                   rinfo, msg_flags,
+                                   ((endTimeStamp - now) > 0) ? (int)((endTimeStamp - now) / 1000) : 0)) > 0) &&
           (offset < bufferLength) ) {
       offset += received;
       if(*msg_flags & MSG_EOR) {
@@ -1645,7 +1644,7 @@ size_t getSessionStatus(struct ComponentAssociation** caeArray,
          session = sessionStorageGetFirstSession(&rserpoolSocket->SessionSet);
          while(session != NULL) {
             if((!session->IsIncoming) &&
-               (session->ConnectedPE != 0)) {
+               (!session->IsFailed)) {
                (*caeArray)[caeArraySize].ReceiverID = CID_COMPOUND(CID_GROUP_POOLELEMENT, session->ConnectedPE);
                (*caeArray)[caeArraySize].Duration   = (session->ConnectionTimeStamp > 0) ? (getMicroTime() - session->ConnectionTimeStamp) : ~0ULL;
                (*caeArray)[caeArraySize].Flags      = 0;
