@@ -1,4 +1,4 @@
-/* $Id: server.cc 2076 2008-12-28 15:52:34Z dreibh $
+/* $Id: server.cc 2122 2009-05-04 07:50:14Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -36,9 +36,6 @@
 #include "fractalgeneratorservice.h"
 #include "calcappservice.h"
 #include "scriptingservice.h"
-#ifdef ENABLE_CSP
-#include "componentstatuspackets.h"
-#endif
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -47,7 +44,7 @@
 #include <math.h>
 #include <sys/types.h>
 #include <signal.h>
-      
+
 
 #define SERVICE_ECHO      1
 #define SERVICE_DISCARD   2
@@ -59,6 +56,29 @@
 #define SERVICE_SCRIPTING 8
 
 
+/* ###### Count the number of CPUs/cores ################################# */
+size_t getNumberOfCPUs()
+{
+   const size_t numberOfCPUs = sysconf(_SC_NPROCESSORS_ONLN);
+/*
+#ifdef __linux__
+   char         buffer[258];
+   unsigned int id;
+   FILE*        fh = fopen("/proc/cpuinfo", "r");
+   if(fh != NULL) {
+
+      while( fgets((char*)&buffer, sizeof(buffer) - 1, fh) != NULL ) {
+         if(sscanf(buffer, "processor\t: %u", &id) == 1) {
+            numberOfCPUs++;
+         }
+      }
+   }
+#endif
+*/
+   return( (numberOfCPUs > 0) ? numberOfCPUs : 1 );
+}
+
+
 /* ###### Main program ################################################### */
 int main(int argc, char** argv)
 {
@@ -489,7 +509,7 @@ start:
       resetStatistics = false;
    }
    else if(service == SERVICE_SCRIPTING) {
-      size_t maxThreads = 1;
+      size_t maxThreads = getNumberOfCPUs();
       ScriptingServer::ScriptingServerSettings settings;
       settings.KeepTempDirs    = false;
       settings.TransmitTimeout = 60000;
