@@ -1,4 +1,4 @@
-/* $Id: tcplikeserver.cc 1726 2008-02-14 09:56:55Z dreibh $
+/* $Id: tcplikeserver.cc 2076 2008-12-28 15:52:34Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -9,7 +9,7 @@
  *
  * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
- * Copyright (C) 2002-2008 by Thomas Dreibholz
+ * Copyright (C) 2002-2009 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -41,14 +41,14 @@
 // ###### Constructor #######################################################
 TCPLikeServer::TCPLikeServer(int rserpoolSocketDescriptor)
 {
-   RSerPoolSocketDescriptor   = rserpoolSocketDescriptor;
-   ServerList                 = NULL;
-   IsNewSession               = true;
-   Shutdown                   = false;
-   Finished                   = false;
-   Load                       = 0;
-   SyncTimerTimeStamp         = 0;
-   AsyncTimerTimeStamp        = 0;
+   RSerPoolSocketDescriptor = rserpoolSocketDescriptor;
+   ServerList               = NULL;
+   IsNewSession             = true;
+   Shutdown                 = false;
+   Finished                 = false;
+   Load                     = 0;
+   SyncTimerTimeStamp       = 0;
+   AsyncTimerTimeStamp      = 0;
    printTimeStamp(stdout);
    printf("New thread for RSerPool socket %d.\n", RSerPoolSocketDescriptor);
 }
@@ -83,9 +83,7 @@ void TCPLikeServer::finishSession(EventHandlingResult result)
 // ###### Shutdown thread ###################################################
 void TCPLikeServer::shutdown()
 {
-   if(!Finished) {
-      Shutdown = true;
-   }
+   Shutdown = true;
 }
 
 
@@ -202,6 +200,7 @@ void TCPLikeServer::run()
          received = rsp_recvfullmsg(RSerPoolSocketDescriptor,
                                     (char*)&buffer, sizeof(buffer),
                                     &rinfo, &flags, (int)(nextTimerEvent / 1000));
+
          if(received > 0) {
             /*
             for(int i = 0;i < received;i++) {
@@ -277,7 +276,7 @@ void TCPLikeServer::poolElement(const char*          programTitle,
                                 struct rsp_info*     info,
                                 struct rsp_loadinfo* loadinfo,
                                 size_t               maxThreads,
-                                TCPLikeServer*       (*threadFactory)(int sd, void* userData),
+                                TCPLikeServer*       (*threadFactory)(int sd, void* userData, const uint32_t peIdentifier),
                                 void                 (*printParameters)(const void* userData),
                                 bool                 (*initializeService)(void* userData),
                                 void                 (*finishService)(void* userData),
@@ -294,7 +293,8 @@ void TCPLikeServer::poolElement(const char*          programTitle,
    }
 
    int rserpoolSocket = rsp_socket(0, SOCK_STREAM, IPPROTO_SCTP);
-   if(rserpoolSocket >= 0) {
+   if( (rserpoolSocket >= 0) &&
+       (rsp_listen(rserpoolSocket, (int)maxThreads) == 0) ) {
       // ====== Initialize notification pipe ================================
       int systemNotificationPipe[2];
       int rspNotificationPipe[2];
@@ -333,10 +333,10 @@ void TCPLikeServer::poolElement(const char*          programTitle,
                   const char* policyName = rsp_getpolicybytype(loadinfo->rli_policy);
                   puts("\n\nGeneral Parameters:");
                   printf("   Pool Handle             = %s\n", poolHandle);
-                  printf("   Reregistration Interval = %u [ms]\n", reregInterval);
+                  printf("   Reregistration Interval = %1.3fs\n", reregInterval / 1000.0);
                   printf("   Runtime Limit           = ");
                   if(runtimeLimit > 0) {
-                     printf("%u [s]\n", runtimeLimit);
+                     printf("%1.3f [s]\n", runtimeLimit / 1000.0);
                   }
                   else {
                      puts("off");
@@ -344,10 +344,10 @@ void TCPLikeServer::poolElement(const char*          programTitle,
                   printf("   Max Threads             = %u\n", (unsigned int)maxThreads);
                   puts("   Policy Settings");
                   printf("      Policy Type          = %s\n", (policyName != NULL) ? policyName : "?");
-                  printf("      Load Degradation     = %1.3f [%%]\n", 100.0 * ((double)loadinfo->rli_load_degradation / (double)PPV_MAX_LOAD_DEGRADATION));
-                  printf("      Load DPF             = %1.3f [%%]\n", 100.0 * ((double)loadinfo->rli_load_dpf / (double)PPV_MAX_LOADDPF));
+                  printf("      Load Degradation     = %1.3f%%\n", 100.0 * ((double)loadinfo->rli_load_degradation / (double)PPV_MAX_LOAD_DEGRADATION));
+                  printf("      Load DPF             = %1.3f%%\n", 100.0 * ((double)loadinfo->rli_load_dpf / (double)PPV_MAX_LOADDPF));
                   printf("      Weight               = %u\n", loadinfo->rli_weight);
-                  printf("      Weight DPF           = %1.3f [%%]\n", 100.0 * ((double)loadinfo->rli_weight_dpf / (double)PPV_MAX_WEIGHTDPF));
+                  printf("      Weight DPF           = %1.3f%%\n", 100.0 * ((double)loadinfo->rli_weight_dpf / (double)PPV_MAX_WEIGHTDPF));
                   if(printParameters) {
                      printParameters(userData);
                   }
@@ -357,9 +357,9 @@ void TCPLikeServer::poolElement(const char*          programTitle,
                if(rsp_register_tags(rserpoolSocket,
                                     (const unsigned char*)poolHandle, strlen(poolHandle),
                                     loadinfo, reregInterval, 0, tags) == 0) {
-                  if(!quiet) {
-                     uint32_t identifier;
-                     if(rsp_getsockname(rserpoolSocket, NULL, NULL, &identifier) == 0) {
+                  uint32_t identifier = 0;
+                  if(rsp_getsockname(rserpoolSocket, NULL, NULL, &identifier) == 0) {
+                     if(!quiet) {
                         puts("Registration");
                         printf("   Identifier              = $%08x\n\n", identifier);
                      }
@@ -385,20 +385,51 @@ void TCPLikeServer::poolElement(const char*          programTitle,
                      // Possible, some sessions have finished working during
                      // waiting time. Therefore, cleaning them up is useful
                      // before accepting new sessions.
-                     serverSet.handleRemovalsAndTimers();
+                     if(serverSet.handleRemovalsAndTimers() > 0) {
+                        int backlog = (int)(serverSet.getMaxThreads() - serverSet.getThreads());
+                        if(rsp_listen(rserpoolSocket, 1 + backlog) < 0) {
+                           perror("Unable to update backlog using rsp_listen()");
+                        }
+                     }
 
                      // ====== Wait for incoming sessions ===================
                      if(result > 0) {
                         if(pollfds[0].revents & POLLIN) {
                             int newRSerPoolSocket = rsp_accept(rserpoolSocket, 0);
                             if(newRSerPoolSocket >= 0) {
-                               TCPLikeServer* serviceThread = threadFactory(newRSerPoolSocket, userData);
-                               if((serviceThread) && (serverSet.add(serviceThread))) {
-                                  serviceThread->start();
+                               if(serverSet.hasCapacity()) {
+                                  TCPLikeServer* serviceThread = threadFactory(newRSerPoolSocket, userData, identifier);
+                                  if(serviceThread) {
+                                     if(serverSet.add(serviceThread)) {
+                                        if(serviceThread->start()) {
+                                           int backlog = (int)(serverSet.getMaxThreads() - serverSet.getThreads());
+                                           if(rsp_listen(rserpoolSocket, 1 + backlog) < 0) {
+                                              perror("Unable to update backlog using rsp_listen()");
+                                           }
+                                        }
+                                        else {
+                                           printTimeStamp(stderr);
+                                           fputs("ERROR: Unable to start up service thread\n", stderr);
+                                           serverSet.remove(serviceThread);
+                                           delete serviceThread;
+                                        }
+                                     }
+                                     else {
+                                        delete serviceThread;
+                                        printTimeStamp(stderr);
+                                        puts("Unable to add new service thread");
+                                     }
+                                  }
+                                  else {
+                                     rsp_close(newRSerPoolSocket);
+                                     printTimeStamp(stderr);
+                                     puts("Unable to create new service thread");
+                                  }
                                }
                                else {
-                                  puts("Rejected new session");
                                   rsp_close(newRSerPoolSocket);
+                                  printTimeStamp(stdout);
+                                  puts("Rejected new session, since server is fully loaded");
                                }
                             }
                         }
@@ -440,6 +471,7 @@ void TCPLikeServer::poolElement(const char*          programTitle,
                   // ====== Clean up ========================================
                   serverSet.removeAll();
                   rsp_deregister(rserpoolSocket, 0);
+                  uninstallBreakDetector();
                }
                if(finishService != NULL) {
                   finishService(userData);
@@ -458,8 +490,8 @@ void TCPLikeServer::poolElement(const char*          programTitle,
          else {
             perror("Unable to map notification pipe FDs");
          }
+         ext_close(systemNotificationPipe[0]);
          ext_close(systemNotificationPipe[1]);
-         ext_close(systemNotificationPipe[2]);
       }
       else {
          perror("Unable to create notification pipe");
@@ -507,28 +539,33 @@ size_t TCPLikeServerList::getThreads()
 
 
 // ###### Remove finished sessions ##########################################
-void TCPLikeServerList::handleRemovalsAndTimers()
+size_t TCPLikeServerList::handleRemovalsAndTimers()
 {
+   size_t removed = 0;
+
    lock();
    ThreadListEntry* entry = ThreadList;
    while(entry != NULL) {
       ThreadListEntry* next = entry->Next;
       if(entry->Object->hasFinished()) {
          remove(entry->Object);
+         removed++;
       }
       else {
-         lock();
+         entry->Object->lock();
          if(entry->Object->AsyncTimerTimeStamp > 0) {
             const unsigned long long now = getMicroTime();
             if(entry->Object->AsyncTimerTimeStamp <= now) {
                entry->Object->asyncTimerEvent(now);
             }
          }
-         unlock();
+         entry->Object->unlock();
       }
       entry = next;
    }
    unlock();
+
+   return(removed);
 }
 
 
