@@ -1,4 +1,4 @@
-/* $Id: tcplikeserver.h 1667 2008-02-04 10:12:05Z dreibh $
+/* $Id: tcplikeserver.h 2086 2009-01-07 10:29:41Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -9,7 +9,7 @@
  *
  * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
- * Copyright (C) 2004-2008 by Thomas Dreibholz
+ * Copyright (C) 2004-2009 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -45,7 +45,7 @@ class TCPLikeServerList : public TDMutex
    ~TCPLikeServerList();
    bool add(TCPLikeServer* thread);
    void remove(TCPLikeServer* thread);
-   void handleRemovalsAndTimers();
+   size_t handleRemovalsAndTimers();
    void removeAll();
 
    double getTotalLoad();
@@ -53,6 +53,9 @@ class TCPLikeServerList : public TDMutex
    inline size_t getMaxThreads() const {
       return(MaxThreads);
    }
+   inline bool hasCapacity() const {
+      return(Threads < MaxThreads);
+   }
 
    private:
    friend class TCPLikeServer;
@@ -104,7 +107,7 @@ class TCPLikeServer : public TDThread
                            struct rsp_info*     info,
                            struct rsp_loadinfo* loadinfo,
                            size_t               maxThreads,
-                           TCPLikeServer*       (*threadFactory)(int sd, void* userData),
+                           TCPLikeServer*       (*threadFactory)(int sd, void* userData, uint32_t peIdentifier),
                            void                 (*printParameters)(const void* userData),
                            bool                 (*initializeService)(void* userData),
                            void                 (*finishService)(void* userData),
