@@ -1,4 +1,4 @@
-/* $Id: udplikeserver.h 1667 2008-02-04 10:12:05Z dreibh $
+/* $Id: udplikeserver.h 2076 2008-12-28 15:52:34Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -9,7 +9,7 @@
  *
  * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
- * Copyright (C) 2002-2008 by Thomas Dreibholz
+ * Copyright (C) 2002-2009 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -42,7 +42,7 @@ class UDPLikeServer
    UDPLikeServer();
    virtual ~UDPLikeServer();
 
-   virtual EventHandlingResult initializeService(int sd);
+   virtual EventHandlingResult initializeService(int sd, const uint32_t peIdentifier);
    virtual void finishService(int sd, EventHandlingResult initializeResult);
 
    virtual void poolElement(const char*          programTitle,
